/*
 *  linux/arch/arm/mach-clps711x/clep7312.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/init.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/sched.h>
#include <linux/mm.h>

#include <asm/hardware.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

extern void clps711x_init_irq(void);
extern void clps711x_map_io(void);

#define MB1 (1024*1024)	/* one megabyte == size of an MMU section */

/*
 * The on-chip registers are given a size of 1MB so that a section can
 * be used to map them; this saves a page table.  This is the place to
 * add mappings for ROM, expansion memory, PCMCIA, etc.  (if static
 * mappings are chosen for those areas).
 *
 * Here is a physical memory map (to be fleshed out later):
 *
 * Physical Address  Size  Description
 * ----------------- ----- ---------------------------------
 * c0000000-c001ffff 128KB reserved for video RAM [1]
 * c0020000-c0023fff  16KB parameters (see Documentation/arm/Setup)
 * c0024000-c0027fff  16KB swapper_pg_dir (task 0 page directory)
 * c0028000-...            kernel image (TEXTADDR)
 *
 * [1] Unused pages should be given back to the VM; they are not yet.
 *     The parameter block should also be released (not sure if this
 *     happens).
 */
static struct map_desc clep7312_io_desc[] __initdata = {
 /* virtual, physical, length, domain, r, w, c, b */

 /* memory-mapped extra keyboard row and CS8900A Ethernet chip */
 { EP7211_VIRT_EXTKBD, EP7211_PHYS_EXTKBD, MB1, DOMAIN_IO, 1, 1, 0, 0 }, 
 { EP7211_VIRT_CS8900A, EP7211_PHYS_CS8900A, MB1, DOMAIN_IO, 1, 1, 0, 0 },

  LAST_DESC
};

static void __init clep7312_map_io(void)
{
        clps711x_map_io();
        iotable_init(clep7312_io_desc);
}

static void __init
fixup_clep7312(struct machine_desc *desc, struct param_struct *params,
	    char **cmdline, struct meminfo *mi)
{
#ifndef CONFIG_USE_PARAM_BLOCK
	mi->nr_banks=1;
	mi->end = 0xc0FFFFFF;
	mi->bank[0].start = 0xc0000000;
	mi->bank[0].size = 0x01000000;
	mi->bank[0].node = 0;
#endif
}


/* mag@sysgo.de: rmk patch defines this as CLEP7212 */
MACHINE_START(EDB7312, "Cirrus Logic 7312")
	MAINTAINER("Nobody")
        BOOT_MEM(0xc0000000, 0x80000000, 0xff000000)
	BOOT_PARAMS(0xc0020100)
	FIXUP(fixup_clep7312)
	MAPIO(clep7312_map_io)
	INITIRQ(clps711x_init_irq)
MACHINE_END

