/*
 * linux/arch/arm/mach-sa1100/dnp1110.c
 *
 * Author: Marco Hasewinkel
 *
 * This file contains all DNP/1110 specific tweaks
 * for "DNP/1110 - mha@ssv <m.hasewinkel@web.de>"
 */
 
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/tty.h>

#include <asm/hardware.h>
#include <asm/setup.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/serial_sa1100.h>

#include "generic.h"

static int __init dnp1110_init(void)
{
	GPDR = 0x08000000;	/* GPIO0-GPIO26 to Input */
				/* GPIO27 to Output */

	GAFR = 0x08000000;	/* alternate function on GPIO27 */

	TUCR = 0x20000000;	/* set 3.6864 MHz out to GPIO27 */

	MECR = 0x1CE71CE7;	/* PCMCIA Socket 0 & 1 Timing */
				/* 3*(7+1)*9,7ns = 233ns - FIXME !!!*/
	return 0;
}

__initcall(dnp1110_init);

#if 0
static void __init
fixup_dnp1110(struct machine_desc *desc, struct param_struct *params,
	  char **cmdline, struct meminfo *mi)
{
	SET_BANK(0, 0xc0000000, 32*1024*1024);	/* DNP/1110 32MB */
//	SET_BANK(0, 0xc0000000, 16*1024*1024);	/* DNP/1110 16MB */
	mi->nr_banks = 1;

	ROOT_DEV = MKDEV(RAMDISK_MAJOR,0);
	setup_ramdisk(1, 0, 0, 8192);
	setup_initrd(0xc0400000, 3*1024*1024);	/* max 3MB INITRAMDISK */
}
#endif

static struct map_desc dnp1110_io_desc[] __initdata = {
 /* virtual     physical    length      domain     r  w  c  b */
  { 0xe8000000, 0x00000000, 0x01000000, DOMAIN_IO, 1, 1, 0, 0 }, /* Flash bank 0 */
  LAST_DESC
};


static void __init dnp1110_map_io(void)
{
	sa1100_map_io();
	iotable_init(dnp1110_io_desc);

	sa1100_register_uart(0, 1);	/* COM1 = ttySA0 (SA UART 1) */
	sa1100_register_uart(1, 3);	/* COM2 = ttySA1 (SA UART 3) */
}

static void __init dnp1110_init_irq(void)
{

/* first default SA11X0 IRQ mapping */
    sa1100_init_irq();

/* then dnp/1110 special IRQ mapping */
    set_GPIO_IRQ_edge(GPIO_GPIO(0), GPIO_RISING_EDGE);	/* USER INT 1 */
    set_GPIO_IRQ_edge(GPIO_GPIO(1), GPIO_RISING_EDGE);	/* USER INT 2 */
    set_GPIO_IRQ_edge(GPIO_GPIO(2), GPIO_RISING_EDGE);	/* USER INT 3 */
    set_GPIO_IRQ_edge(GPIO_GPIO(3), GPIO_RISING_EDGE);	/* USER INT 4 */
    set_GPIO_IRQ_edge(GPIO_GPIO(4), GPIO_RISING_EDGE);	/* USER INT 4 */

    set_GPIO_IRQ_edge(GPIO_GPIO(5), GPIO_RISING_EDGE);	/* LAN */
    set_GPIO_IRQ_edge(GPIO_GPIO(6), GPIO_RISING_EDGE);	/* SSV only */
}

MACHINE_START(DNP1110, "DNP/1110 - SSV Embedded Systems")
	MAINTAINER("mha@ssv - <m.hasewinkel@web.de>")
	BOOT_MEM(0xc0000000, 0x80000000, 0xf8000000)
	BOOT_PARAMS(0xc0000100)
/*	FIXUP(fixup_dnp1110) */
	MAPIO(dnp1110_map_io)
	INITIRQ(dnp1110_init_irq)
MACHINE_END
