/*
 * arch/ppc/8xx_io/watchdog.c
 * 
 * MPC8xx internal watchdog reset
 *
 * Copyright 2002 SYSGO Realtime Solutions
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/time.h>
#include <linux/sched.h>
#include <linux/timer.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/bootinfo.h>
#include <asm/mpc8xx.h>
#include <asm/machdep.h>
#include <asm/todc.h>
#include <asm/commproc.h>

static struct timer_list mpc8xx_wd_timer;

/*
 * reset the internal watchdog timer
 */
void mpc8xx_wd_ping(void)
{
		volatile immap_t	*immap;

		immap = (immap_t *)IMAP_ADDR;

		immap->im_siu_conf.sc_swsr = 0x556c;   /* write magic1 */
		immap->im_siu_conf.sc_swsr = 0xaa39;   /* write magic2 */
}

/*
 * watchdog timer function
 */
static void mpc8xx_wd_timerfunc(unsigned long d)
{
		mpc8xx_wd_ping();
		/* re-start the timer for 200ms */
		mod_timer(&mpc8xx_wd_timer, jiffies + HZ/5);
}

/*
 * start the watchdog timer
 */
void mpc8xx_wd_timer_start(void)
{
		init_timer(&mpc8xx_wd_timer);
		mpc8xx_wd_timer.expires = jiffies + HZ/5; /* 200ms */
		mpc8xx_wd_timer.function = mpc8xx_wd_timerfunc;
		mpc8xx_wd_timer.data = 0;
		add_timer(&mpc8xx_wd_timer);
		mpc8xx_wd_ping();
		printk(KERN_INFO "MPC8xx watchdog reset timer started.\n");
}

