/*
 * arch/ppc/kernel/men824xali_pci.c
 * 
 * PCI support for MEN F1
 *
 * Author: Matt Porter <mporter@mvista.com>
 *
 * Copyright 2001 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/malloc.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/pci.h>
//#include "pci_auto.h"
#include <asm/mpc10x.h>

#include "men824xali.h"

#define DEBUG
#ifdef DEBUG
#define DBG(x...) printk(x)
#else
#define DBG(x...)
#endif /* DEBUG */ 

struct pci_controller* men824xali_hose;

void __init
men824xali_find_bridges(void)
{
	struct pci_controller* hose;
	unsigned char pin;
	int rv;

	//printk("MEN 824XALI men824xali_find_bridges\n");

	men824xali_hose = hose = pcibios_alloc_controller();
	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	rv = mpc10x_bridge_init(hose,
			MPC10X_MEM_MAP_B,
			MPC10X_MEM_MAP_B,
			MPC10X_MAPB_EUMB_BASE);
    
	/*
	 * popp@men.de:
	 * Limit the PCI I/O space usage to 64k. Some devices (like the P10),
	 * can decode only 16 bits of the PCI address range. 
	 * 
	 * Additionally, do not reassign space that are covered by fixed devices
	 * (ALI) 
	 */
	hose->io_space.start 	= 0x2200;
	hose->io_space.end 		= 0xefff;

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);
	
	{
		/* Add ISA bus wait states */
		unsigned char isa_control;

		early_read_config_byte(hose, 0, 0x90, 0x43, &isa_control);
		isa_control |= 0x33;
		early_write_config_byte(hose, 0, 0x90, 0x43, isa_control);
	}


	/* Early configure PCI access to PMU for I2C_RTC */
	ali15x3_setup_early(hose, 0x1C);

	/*
	 * popp@men.de:
	 * do not reassign PCI interrupts. Keep MENMON settings. 
	 */
	/*	ppc_md.pci_swizzle = common_swizzle;*/
	/*	ppc_md.pci_map_irq = men824xali_map_irq; kp*/
}
