/*
 * arch/ppc/platforms/men824xali_setup.c
 * 
 * Board setup routines for MEN 824XALI
 *
 * Original Author: Matt Porter <mporter@mvista.com>
 *
 * Reimplemented 2002 by Marius Groeger <mgroeger@sysgo.de>, 
 * and Klaus Popp <klaus.popp@men.de>
 *
 * Copyright 2001 MontaVista Software Inc.
 * Copyright 2002 MEN Mikro Elektronik GmbH, SYSGO realtime solutions
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <asm/serial.h>

#include <asm/keyboard.h>
#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/time.h>
#include <asm/i8259.h>
#include <asm/open_pic.h>
#include <asm/bootinfo.h>
#include <asm/mpc10x.h>
#include <asm/todc.h>

#include "men824xali.h"
#include "men824xali_time.h"
#include "mmparastring.h"

#ifdef CONFIG_I2C_M41T56
extern int i2c_init(void);
extern int i2c_ali15x3_init(void); 
extern int m41t56_init(void);
#endif

#ifdef CONFIG_VT
extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
		                           char raw_mode);
extern char pckbd_unexpected_up(unsigned char keycode);
extern void pckbd_leds(unsigned char leds);
extern void pckbd_init_hw(void);
extern unsigned char pckbd_sysrq_xlate[128];
#endif /* CONFIG_VT */

extern void men824xali_find_bridges(void);
extern unsigned long loops_per_jiffy;
extern char cmd_line[];
char men824xali_mmparastring[512];
#if defined(CONFIG_MEN824XALI_WATCHDOG) && !defined(CONFIG_MENF1N)
extern int men824xwd_enable_ping_early;
#endif

#define ALI1543_PMU_DEV_NO      0x1C

/* ALI PMU register offsets */
#define ALI1543_PMU_DOGPO1      0xC0   
#define ALI1543_PMU_GPIO_DIR 	0x7d
#define ALI1543_PMU_GPIO_DOUT 	0x7e
#define ALI1543_PMU_GPIO_DIN 	0x7f

#ifdef CONFIG_MENB11
#  define GPIO_RESET		0x02 	/* GPIO 1 */
#endif

#ifdef CONFIG_MENA12
#  define GPIO_RESET		0x01 	/* Rev.0 boards: GPIO 0 */
#  define GPO_RESET			0x08	/* Rev.1 boards: GPO 3 */
#endif

#ifdef CONFIG_VGA_CONSOLE
#include <asm/vga.h>
/* 
 * The following VGA functions are taken from arch/ppc/boot/prep/vreset.c
 */

/*
 * VGA Register
 */
struct VgaRegs
{
	unsigned short io_port;
	unsigned char  io_index;
	unsigned char  io_value;
};

/*
 * Default console text mode registers  used to reset
 * graphics adapter.
 */
#define NREGS 54
#define ENDMK  0xFFFF  /* End marker */

static struct __initdata VgaRegs GenVgaTextRegs[NREGS+1] = {
	/* port		index	value  */
	/* SR Regs */
	{ 0x3c4,	0x1,	0x0 },
	{ 0x3c4,	0x2,	0x3 },
	{ 0x3c4,	0x3,	0x0 },
	{ 0x3c4,	0x4,	0x2 },
	 /* CR Regs */
	{ 0x3d4,	0x0,	0x5f },
	{ 0x3d4,	0x1,	0x4f },
	{ 0x3d4,	0x2,	0x50 },
	{ 0x3d4,	0x3,	0x82 },
	{ 0x3d4,	0x4,	0x55 },
	{ 0x3d4,	0x5,	0x81 },
	{ 0x3d4,	0x6,	0xbf },
	{ 0x3d4,	0x7,	0x1f },
	{ 0x3d4,	0x8,	0x00 },
	{ 0x3d4,	0x9,	0x4f },
	{ 0x3d4,	0xa,	0x0d },
	{ 0x3d4,	0xb,	0x0e },
	{ 0x3d4,	0xc,	0x00 },
	{ 0x3d4,	0xd,	0x00 },
	{ 0x3d4,	0xe,	0x00 },
	{ 0x3d4,	0xf,	0x00 },
	{ 0x3d4,	0x10,	0x9c },
	{ 0x3d4,	0x11,	0x8e },
	{ 0x3d4,	0x12,	0x8f },
	{ 0x3d4,	0x13,	0x28 },
	{ 0x3d4,	0x14,	0x1f },
	{ 0x3d4,	0x15,	0x96 },
	{ 0x3d4,	0x16,	0xb9 },
	{ 0x3d4,	0x17,	0xa3 },
	 /* GR Regs */
	{ 0x3ce,	0x0,	0x0 },
	{ 0x3ce,	0x1,	0x0 },
	{ 0x3ce,	0x2,	0x0 },
	{ 0x3ce,	0x3,	0x0 },
	{ 0x3ce,	0x4,	0x0 },
	{ 0x3ce,	0x5,	0x10 },
	{ 0x3ce,	0x6,	0xe },
	{ 0x3ce,	0x7,	0x0 },
	{ 0x3ce,	0x8,	0xff },
	{ ENDMK }
};

static unsigned char __initdata AC[21] = {
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x14, 0x07,
    0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
    0x0C, 0x00, 0x0F, 0x08, 0x00};
    
static unsigned char __initdata *ISA_io = NULL;

#undef outb
#undef inb
/* The original outb()/inb() from asm/io.h don't work */
static void __init
outb(int port, unsigned char val)
{
	/* Ensure I/O operations complete */
	__asm__ volatile("eieio");
	ISA_io[port] = val;
}

static u8 __init
inb(int port)
{
	/* Ensure I/O operations complete */
	__asm__ volatile("eieio");
	return (ISA_io[port]);
}

/*
 * Write to VGA Attribute registers.
 */
static void __init
writeAttr(unsigned char index, unsigned char data, unsigned char videoOn)
{

	unsigned char v;
	v = inb(0x3da);   /* reset attr. address toggle */
	if (videoOn)
		outb(0x3c0, (index & 0x1F) | 0x20);
	else
		outb(0x3c0, (index & 0x1F));
	outb(0x3c0, data);
}

static void __init
setTextRegs(struct VgaRegs *svp)
{
	int i;
	/*
	 *  saved settings
	 */
	while( svp->io_port != ENDMK ) {
		outb(svp->io_port,   svp->io_index);
		outb(svp->io_port+1, svp->io_value);
		svp++;
	}

	outb(0x3c2, 0x67);  /* MISC */
	outb(0x3c6, 0xff);  /* MASK */

	for ( i = 0; i < 0x10; i++)
		writeAttr(i, AC[i], 0);  /* pallete */

	writeAttr(0x10, 0x0c, 0);    /* text mode */
	writeAttr(0x11, 0x00, 0);    /* overscan color (border) */
	writeAttr(0x12, 0x0f, 0);    /* plane enable */
	writeAttr(0x13, 0x08, 0);    /* pixel panning */
	writeAttr(0x14, 0x00, 1);    /* color select; video on  */
}
#endif

/*
 * Interrupt vectors used by MEN 824x/ALI boards:
 * 0..15	8259 PIC
 * 16..20	EPIC external IRQs (IRQ0..5) IRQ0 cascaded to 8259 PIC
 * 32..33	EPIC internal IRQs (DUART0/1)
 * 64..67	EPIC timers
 * 127		EPIC spurious
 */

#define MEN824XALI_PIC_CASCADE_VEC 16
#define MEN824XALI_PIC_DUART0      137
#define MEN824XALI_PIC_DUART1      138

/*
 * *** IMPORTANT ***
 *
 * Array to define the interrupt sense and polarity of the EPIC (OpenPic)
 * sources.
 *
 * The first 16 entries of 'men824xali_openpic_initsenses[]' are there and
 * initialized to 0 on purpose.  DO NOT REMOVE THEM as the 'offset' parameter
 * of 'openpic_init()' does not work for us because the EPIC's IRQ 0's offset 
 * is the same as a normal openpic's IRQ 16 offset.
 *
 * There are only two choices:
 * 0: edge sensitve (rising edge)
 * 1: level sensitive (low active)
 * 
 * Note: polarity of IRQ0 is patched later in men824xali_init_IRQ()
 * to high active
 */
static u_char men824xali_openpic_initsenses[] __initdata = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	1,   /* EPIC IRQ0: PIC 8259 cascade interrupt, high active! */
	1,   /* EPIC IRQ1: not yet used */
	1,   /* EPIC IRQ2: not yet used */
	1,   /* EPIC IRQ3: not yet used */
	1,   /* EPIC IRQ4: not yet used */
};


static int
men824xali_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "machine\t\t: MEN 824XALI "
#ifdef CONFIG_MENA12
			   "A12/D3/SC13"
#endif
#ifdef CONFIG_MENB11
			   "B11"
#endif
#ifdef CONFIG_MENF1N
			   "F1N"
#endif
			   "\n");
	return 0;
}


static void __init
men824xali_setup_arch(void)
{
	/* init to some ~sane value until calibrate_delay() runs */
	loops_per_jiffy = 50000000/HZ;
	
	/* Lookup PCI host bridges */
	men824xali_find_bridges();
	
#ifdef CONFIG_ROOT_NFS
	ROOT_DEV = to_kdev_t(0x00ff); /* /dev/nfs pseudo device */
#else
	ROOT_DEV = to_kdev_t(0x1601); /* /dev/hdc1 */
#endif
#ifdef CONFIG_I2C_M41T56
	/* 
	 * this is needed for time_init(). 
	 * Otherwise "real time clock seems stuck" and date is 1.1.1970
	 *
	 * pba@sysgo.de: Initialize just necesary parts of i2c
	 * Otherwise
	 *  - in the 2nd initialization: "i2c-dev.o: Adapter unregistered: SMBus ALI15X3 adapter at eee0"
	 *  - with devfs kernel hangs here
	 *  - with CONFIG_I2C_PROC kernel hangs here
	 */
	i2c_init();
	i2c_ali15x3_init();
	m41t56_init();
#endif

#ifdef CONFIG_VGA_CONSOLE
	/*
	 * pba@sysgo.de:
	 * MenMon on MEN A12 board locks attribute memory access.
	 * So we reload all registers to default values
	 */ 
	ISA_io = (unsigned char *) _IO_BASE;
	setTextRegs(GenVgaTextRegs);     		
	/* pba@sysgo.de: ISA memory base, VGA console functions switch */
	vgacon_remap_base = _ISA_MEM_BASE;
	conswitchp = &vga_con;
#endif

}

/*
 * Use the GPIOs or GPOs to reset board. Unfortunately, the boards
 * use different pins (the pins may even differ through different
 * revisions)
 */
static void
men824xali_restart(char *cmd)
{
	int aliDevFn = PCI_DEVFN(ALI1543_PMU_DEV_NO,0);
	u8 reg;
	extern struct pci_controller* men824xali_hose;

	__cli();
	

#if defined(CONFIG_MENA12) || defined(CONFIG_MENB11) 

# if defined(CONFIG_MENA12)
	/* A12/D3/SC13 revision 1 boards: GPO3 is used to reset board */
	early_read_config_byte( men824xali_hose, 0, aliDevFn, 
							ALI1543_PMU_DOGPO1, &reg );
	early_write_config_byte( men824xali_hose, 0, aliDevFn, 
							 ALI1543_PMU_DOGPO1, reg & ~GPO_RESET );

	/* if still no reset, it's propably a rev. 0 board, fall through */

# endif /* A12/D3/SC13 */	

	/* program GPIO to low */
	early_read_config_byte( men824xali_hose, 0, aliDevFn, 
							ALI1543_PMU_GPIO_DOUT, &reg );
	early_write_config_byte( men824xali_hose, 0, aliDevFn, 
							 ALI1543_PMU_GPIO_DOUT, reg & ~GPIO_RESET );

	/* program GPIO as output */
	early_read_config_byte( men824xali_hose, 0, aliDevFn, 
							ALI1543_PMU_GPIO_DIR, &reg );
	early_write_config_byte( men824xali_hose, 0, aliDevFn, 
							 ALI1543_PMU_GPIO_DIR, reg | GPIO_RESET );
#endif /*  A12/D3/SC13/B11 */	


#if defined(CONFIG_MENF1N)
	/* start watchdog, but don't service it, reset takes approx. 0.25s */
	/* ! don't use TODC_TYPE_MK48T59_WATCHDOG from todc.h. It's wrong! */
	todc_m48txx_write_val( 0x1ff7, 0x85	);
#endif /* F1N*/

	while(1);
}

static void
men824xali_halt(void)
{
	__cli();
	while (1);
}

static void
men824xali_power_off(void)
{
	men824xali_halt();
}

static void __init
men824xali_init_IRQ(void)
{
	int i;
	unsigned long pvr;
	
	/* 
	 * setup interrupts and other parameters for DUART units 
	 * The two additional UARTs are available on A12 only. Additionally
	 * they are only present on 8245 (not 8240)
	 */
	pvr = mfspr(PVR);
#ifdef CONFIG_MENA12
	if( PVR_VER(pvr) == PVR_VER(PVR_8245) )
	{
		char strBuf[20];		
		struct serial_struct serial_req;
		unsigned long bus_clock = 100000000UL;
		void *DUART_Addr;

		printk("men824xali_setup: detected 8245, init DUARTs\n");		
		/* if possible, use bus clock as reported by MENMON */
		if( MMPARA_SearchKey("memclk=", men824xali_mmparastring, strBuf)) {	
			if( *MMPARA_StringToInt(10, strBuf, &bus_clock) == '\0'){
			    printk(" Using bus clock from MENMON %ldMHz\n", bus_clock );
			    bus_clock *= 1000000UL;	/* bus_clock is expressed in MHz */			    
			}
		}
	
		DUART_Addr = ioremap(MPC10X_MAPB_EUMB_BASE + MPC10X_EUMB_DUART_OFFSET,
				MPC10X_EUMB_DUART_SIZE);
				
		writeb(0x01, DUART_Addr + 0x511);  /* select DUART mode (DCR register) */
		
		memset(&serial_req, 0, sizeof(serial_req));		
		serial_req.line = 2;
		serial_req.baud_base = bus_clock/16;
		serial_req.irq = MEN824XALI_PIC_DUART0;
		serial_req.flags = STD_COM_FLAGS;
		serial_req.io_type = SERIAL_IO_MEM;
		serial_req.iomem_base = DUART_Addr + 0x0500;
		serial_req.iomem_reg_shift = 0;
		if (early_serial_setup(&serial_req) != 0) {
			printk("Early serial init of DUART0 failed\n");
		}
		
		/* Assume early_serial_setup() doesn't modify serial_req */
		serial_req.line = 3;
		serial_req.irq = MEN824XALI_PIC_DUART1;
		serial_req.iomem_base = DUART_Addr + 0x0600;
		if (early_serial_setup(&serial_req) != 0) {
			printk("Early serial init of DUART1 failed\n");
		}
		
		openpic_set_sources(0, 139, NULL);		
	}
#endif
	
	/*
	 * setup interrupt driver for 824x (using code in ../kernel/open_pic.c)
	 * OpenPIC_InitSenses and OpenPIC_NumInitSenses are global variables
	 * in open_pic.c
	 *
	 * openpic_init takes four parameters:
	 * - main_pic (bool) if false, doesn't do all the initialisation,
	 *            so we pass true
	 * - offset - sets up the global open_pic_irq_offset, offset of
	 *			  first irq number handled by open pic (see comment in 
	 *			  initsenses above)
	 * - chrp_ack - not used as long we don't call openpic_get_irq() 
	 * - programmer_switch_irq - if >=0 defines an external irq that has
	 *			  priority 9, all others have prio 8. 
	 *
	 */
	OpenPIC_InitSenses = men824xali_openpic_initsenses;
	OpenPIC_NumInitSenses = sizeof(men824xali_openpic_initsenses);

	openpic_init(1, 0, NULL, -1);
	
	/*
	 * Hack alert!!!
	 * Need to patch the polarity of IRQ0 to be high active after
	 * openpic_init() has set it to low active. Required since initsenses
	 * has only two choises: low active/level or high active/edge
	 */
	{
		u_int irq0reg;

		irq0reg = in_le32( (volatile u_int *)((char *)OpenPIC_Addr + 0x10200));
		irq0reg |= 1<<23;
		out_le32( (volatile u_int *)((char *)OpenPIC_Addr + 0x10200), irq0reg);
	}

	/*
	 * open_pic has initialized entries 0..26
	 * initialize the first 16 entries to be handled by 8259 PIC
	 */
	for ( i = 0 ; i < NUM_8259_INTERRUPTS  ; i++ )
		irq_desc[i].handler = &i8259_pic;

	if (request_irq(MEN824XALI_PIC_CASCADE_VEC, no_action, SA_INTERRUPT,
			"8259 cascade to EPIC", NULL)) {
	
		printk("Unable to get OpenPIC IRQ %d for cascade\n",
			MEN824XALI_PIC_CASCADE_VEC);
	}

	i8259_init(0xfef01ff0);
}

static int men824xali_get_irq(struct pt_regs *regs)
{
	int irq = openpic_irq();

	/*printk("*** get_irq EPIC %d\n", irq );*/

	/* if EPIC irq was IRQ0, ask 8259 PIC for vector */
	if( irq == MEN824XALI_PIC_CASCADE_VEC ){
		irq = i8259_irq();

		/* ??? is it a good idea to EOI EPIC before PIC is EOI'd ? */
		openpic_eoi();
	}
	else if( irq == OPENPIC_VEC_SPURIOUS )
		irq = -1;

	/*printk("**** get_irq %d\n", irq );*/
	return irq;
}

/*
 * Set BAT 3 to map 0xFE000000.
 */ 
static __inline__ void
men824xali_set_bat(void)
{
	unsigned long   bat3u, bat3l;
	static int	mapping_set = 0;

	if (!mapping_set)
	{
		__asm__ __volatile__(
				" lis %0,0xFE00\n \
				ori %1,%0,0x002a\n \
				ori %0,%0,0x1ffe\n \
				mtspr 0x21e,%0\n \
				mtspr 0x21f,%1\n \
				isync\n \
				sync "
				:: "r" (bat3u), "r" (bat3l));

		mapping_set = 1;
	}
	return;
}

unsigned long __init
men824xali_find_end_of_memory(void)
{
	unsigned long memSize = 0;
	char strBuf[20];

	/* Cover the I/O with a BAT */
	men824xali_set_bat();

	/* if possible, use memory size as reported by MENMON */
	if( MMPARA_SearchKey("mem0=", men824xali_mmparastring, strBuf)){
	
		if( *MMPARA_StringToInt(10, strBuf, &memSize) == '\0'){
			memSize *= 1024;	/* mem0 is expressed in kBytes */
			printk("Using memory size from MENMON 0x%lx\n", memSize );
		}
	}

	/* if mem0 was not present in menmon parameter string, ask 824x */
	if( memSize == 0 ){
		/* Read the memory size from the MPC107 */
		memSize = mpc10x_get_mem_size(MPC10X_MEM_MAP_B);
	}
	return memSize;	
}

static void __init
men824xali_map_io(void)
{
    	/* also contains EUMB (mapped to MPC10X_MAPB_EUMB_BASE) */
	io_block_mapping(0xfd000000, 0xfd000000, 0x02000000, _PAGE_IO);
	io_block_mapping(0x80000000, 0x80000000, 0x20000000, _PAGE_IO);
#if defined(CONFIG_MEN824XALI_WATCHDOG) && !defined(CONFIG_MENF1N)
	/* we have now mapped PCI cfg. space, we can togle WD pin in printk() */
	men824xwd_enable_ping_early = 1;	
#endif	
}


#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)
int
men824xali_ide_check_region(ide_ioreg_t from, unsigned int extent)
{
        return check_region(from, extent);
}

void
men824xali_ide_request_region(ide_ioreg_t from,
			unsigned int extent,
			const char *name)
{
        request_region(from, extent, name);
}

void
men824xali_ide_release_region(ide_ioreg_t from,
			unsigned int extent)
{
        release_region(from, extent);
}

void __init
men824xali_ide_init_hwif_ports (hw_regs_t *hw, ide_ioreg_t data_port,
		ide_ioreg_t ctrl_port, int *irq)
{
	ide_ioreg_t reg = data_port;
	int i = 8;

	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw->io_ports[i] = reg;
		reg += 1;
	}
	if (ctrl_port)
		hw->io_ports[IDE_CONTROL_OFFSET] = ctrl_port;
	else
		hw->io_ports[IDE_CONTROL_OFFSET] =
			hw->io_ports[IDE_DATA_OFFSET] + 0x206;

	if (irq != NULL)
		*irq = 0;
}

static int
men824xali_ide_default_irq(ide_ioreg_t base)
{
	if (base == MEN824XALI_IDE0_BASE_ADDR)
		return 14;
	else if (base == MEN824XALI_IDE1_BASE_ADDR)
		return 15;
	else
		return 0;
} 

static ide_ioreg_t
men824xali_ide_default_io_base(int index)
{
	if (index == 0)
		return MEN824XALI_IDE0_BASE_ADDR;
	else if (index == 1)
		return MEN824XALI_IDE1_BASE_ADDR;
	else
		return 0;
}                                                                               
#endif

/*
 * This sets up global var men824xali_mmparastring using the boot info
 * tag setup by boot/common/misc-simple.c. It contains an ASCII string
 * with key/value pairs from MENMON 
 */
void
men824xali_find_mmparastring(struct bi_record *rec)
{
	if (rec == NULL || rec->tag != BI_FIRST)
		return;
	while (rec->tag != BI_LAST) {
		ulong *data = rec->data;
		switch (rec->tag) {
		case BI_MENMON_PARAMETERS:
			memcpy(men824xali_mmparastring, (void *)data, rec->size);
			break;
		}
		rec = (struct bi_record *)((ulong)rec + rec->size);
	}
}

#if defined(CONFIG_MENF1N)
TODC_ALLOC();
#endif

void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	  unsigned long r6, unsigned long r7)
{
	struct bi_record *bi;
	char strBuf[20];

	/*
	 * If we have been started from Linux pre booter, then
	 * the bootinfo records are placed exactly at __end (end of BSS
	 * segment. In this case, find_bootinfo() will find the bootinfo
	 * records.
	 *
	 * However, if we have been started from MENMON's PPCBOOT compatible 
	 * boot method, then r3 points to the physical address of the
	 * bootinfo structure
	 */
	 
	if( (bi = find_bootinfo()) == NULL ) {
	    if (r3 != 0)
			bi = (struct bi_record *)(r3 + KERNELBASE);	    
	    /* be more paranoid - check for BI_FIRST and correct size */
	    if (!( ( bi->tag == BI_FIRST ) && 
			   ( bi->size == sizeof(struct bi_record)))) {	
			printk("Bootinfo records not found\n");	    
			bi = NULL;;
	    }
	}    
	
	parse_bootinfo(bi);
	    
	/* get the MENMON parameter string */
	men824xali_find_mmparastring( bi );
		    
	printk("MENMON parameter string: %s\n",men824xali_mmparastring); 
	/*printk("MEN 824XALI INIT!\n\n");*/

	/* Look for console= option on command line */
	if (strstr(cmd_line, "console=") == 0) {
		/* 
		 * if no console= on the command line, then use the 
		 * menmon parameters to add one 
		 */
		if (MMPARA_SearchKey("cons=", men824xali_mmparastring, strBuf) != NULL)
		{
			if (strcmp(strBuf, "COM1") == 0)
			{
				strcat( cmd_line, " console=ttyS0");
				if (MMPARA_SearchKey("cbr=", men824xali_mmparastring, strBuf) 
					!= NULL)
				{
					strcat((char *) strcat((char *) &cmd_line, ","), strBuf);
				}
			}
			else if (strcmp(strBuf, "VGA") == 0)
			{
				strcat( cmd_line, " console=tty");
			}
		}
	}
	printk("Command line now %s\n", cmd_line );

	isa_io_base = MPC10X_MAPB_ISA_IO_BASE;
	isa_mem_base = MPC10X_MAPB_ISA_MEM_BASE;
	pci_dram_offset = MPC10X_MAPB_DRAM_OFFSET;

	ppc_md.setup_arch	= men824xali_setup_arch;
	ppc_md.show_cpuinfo	= men824xali_show_cpuinfo;
	ppc_md.init_IRQ		= men824xali_init_IRQ;
	ppc_md.get_irq		= men824xali_get_irq;

	ppc_md.find_end_of_memory = men824xali_find_end_of_memory;
	ppc_md.setup_io_mappings = men824xali_map_io;
	
	ppc_md.restart		= men824xali_restart;
	ppc_md.power_off	= men824xali_power_off;
	ppc_md.halt		= men824xali_halt;

	ppc_md.progress = NULL;

#if defined(CONFIG_MENB11) || defined(CONFIG_MENA12) 
	ppc_md.time_init	= men824xali_time_init;
	ppc_md.get_rtc_time	= men824xali_get_rtc_time;
	ppc_md.set_rtc_time	= men824xali_set_rtc_time;
	ppc_md.calibrate_decr	= men824xali_calibrate_decr;

	ppc_md.nvram_read_val   = men824xali_rtc_read_val;
	ppc_md.nvram_write_val  = men824xali_rtc_write_val;
#endif

#if defined(CONFIG_MENF1N) 
	TODC_INIT(TODC_TYPE_MK48T59,
		  MEN824XALI_NVRAM_AS0,
		  MEN824XALI_NVRAM_AS1,
		  MEN824XALI_NVRAM_DATA,
		  7);

	ppc_md.nvram_read_val	= todc_m48txx_read_val;
	ppc_md.nvram_write_val	= todc_m48txx_write_val;

	ppc_md.time_init	= todc_time_init;
	ppc_md.get_rtc_time	= todc_get_rtc_time;
	ppc_md.set_rtc_time	= todc_set_rtc_time;
	ppc_md.calibrate_decr	= todc_calibrate_decr;
#endif


#ifdef CONFIG_VT
	/*
	 * Support for PS/2 keyboard
	 */
	ppc_md.kbd_setkeycode    = pckbd_setkeycode;
	ppc_md.kbd_getkeycode    = pckbd_getkeycode;
	ppc_md.kbd_translate     = pckbd_translate;
	ppc_md.kbd_unexpected_up = pckbd_unexpected_up;
	ppc_md.kbd_leds          = pckbd_leds;
	ppc_md.kbd_init_hw       = pckbd_init_hw;
# ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.ppc_kbd_sysrq_xlate = pckbd_sysrq_xlate;
	SYSRQ_KEY = 0x54;
# endif
#endif /* CONFIG_VT */

#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)
	ppc_ide_md.default_io_base = men824xali_ide_default_io_base;
	ppc_ide_md.default_irq = men824xali_ide_default_irq;
	ppc_ide_md.ide_check_region = men824xali_ide_check_region;
	ppc_ide_md.ide_request_region = men824xali_ide_request_region;
	ppc_ide_md.ide_release_region = men824xali_ide_release_region;
	ppc_ide_md.ide_init_hwif = men824xali_ide_init_hwif_ports;
#endif
}
