#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/time.h>
#include <linux/i2c.h>
#include <linux/m41t56rtc.h>

#include <asm/time.h>
#include <asm/todc.h>

#include "men824xali_time.h"
#include "mmparastring.h"

extern unsigned long
mktime(unsigned int year, unsigned int mon, unsigned int day,
       unsigned int hour, unsigned int min, unsigned int sec);
       
static struct i2c_client *rtc_client;

static inline int rtc_command(int cmd, void *data)
{
	int ret = -EIO;

	if (rtc_client)
		ret = rtc_client->driver->command(rtc_client, cmd, data);

	/*printk("rtc_command: ret=%d\n", ret );*/
	return ret;
}


u_char
men824xali_rtc_read_val(int addr)
{	
	u_char ret;	
	struct mem rtc_mem = {addr, 1, &ret};
	
	if (rtc_command(MEM_READ, &rtc_mem))
		return 0;
	
	return ret;
}
  
void
men824xali_rtc_write_val(int addr, unsigned char val)
{
	struct mem rtc_mem = {addr, 1, &val};
	
	rtc_command(MEM_WRITE, &rtc_mem);		
}

ulong
men824xali_get_rtc_time(void)
{
	struct rtc_tm rtctm;		
	
	if (rtc_command(RTC_GETDATETIME, &rtctm))
		return 0;

	if (rtctm.year < 70)
		rtctm.year += 100;

	return mktime(1900 + rtctm.year, rtctm.mon, rtctm.mday,
			       rtctm.hours, rtctm.mins, rtctm.secs);	
}


int
men824xali_set_rtc_time(unsigned long nowtime)
{	
	struct rtc_tm new_rtctm, old_rtctm;
	struct rtc_time new_compat_rtctm; 
	
	to_tm(nowtime, &new_compat_rtctm);
	
       /*
        * pba@sysgo.de: new_rtctm.year is char ->  we must substract epoch from new_compat_rtctm.tm_year
        */
	new_rtctm.year = new_compat_rtctm.tm_year - 1900;
	new_rtctm.mon = new_compat_rtctm.tm_mon;
	new_rtctm.hours = new_compat_rtctm.tm_hour;
	new_rtctm.mins = new_compat_rtctm.tm_min;
	new_rtctm.secs = new_compat_rtctm.tm_sec;
	new_rtctm.wday = new_compat_rtctm.tm_wday;
	new_rtctm.mday = new_compat_rtctm.tm_mday;
	

	if (rtc_command(RTC_GETDATETIME, &old_rtctm))
		return -1;	
	
	/*
	 * avoid writing when we're going to change the day
	 * of the month.  We will retry in the next minute.
	 * This basically means that if the RTC must not drift
	 * by more than 1 minute in 11 minutes.
	 *
	 * [ rtc: 1/1/2000 23:58:00, real 2/1/2000 00:01:00,
	 *   rtc gets set to 1/1/2000 00:01:00 ]
	 */
	if ((old_rtctm.hours == 23 && old_rtctm.mins  == 59) ||
	    ((u_char)new_compat_rtctm.tm_hour == 23 && (u_char)new_compat_rtctm.tm_min  == 59))
		return -1;

	/* 
	 * pba@sysgo.de: RTC_SETTIME -> RTC_SETDATETIME 
	 */
	return rtc_command(RTC_SETDATETIME, &new_rtctm);	
}


/*
 * Read MENMON parameter string, determine decrementer clock from
 * "memclk="
 */
void __init
men824xali_calibrate_decr(void)
{
	ulong	freq;
	extern char men824xali_mmparastring[];
	char strBuf[20];

	if( MMPARA_SearchKey( "memclk=", men824xali_mmparastring, strBuf ) 
		!= NULL ){

		MMPARA_StringToInt(10, strBuf, &freq);
		/* decrementer clock = memory clock / 4 */	
		freq *= 1000000 / 4; 
	}	
	else {
		printk("*** NO memclk= in menmon parameter string!\n");
		freq = 25000000;
	}
	printk("time_init: decrementer frequency = %lu.%.6lu MHz\n",
	       freq/1000000, freq%1000000);

	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
	
	return;
}

/* Initialisation = recherche de la structure cliente RTC M41T56 sur ALI15X3 */
long __init
men824xali_time_init(void)
{
	rtc_client = i2c_get_client(I2C_DRIVERID_M41T56, 
								I2C_ALGO_SMBUS|I2C_HW_SMBUS_ALI15X3, NULL);	
	return 0;	
}
