/*********************  P r o g r a m  -  M o d u l e ***********************
 *  
 *         Name: mmparastring.c
 *      Project: many
 *
 *       Author: kp
 *        $Date: 2001/04/25 08:44:43 $
 *    $Revision: 1.3 $
 *
 *  Description: Routines to parse MENMON parameter string
 *                      
 *	Some MENMONs provide an ASCII string in memory containing board and
 *	other parameters (first implemented on MEN B11). This library allows
 *	you to parse these parameters easily. Can be used also to parse parameters
 *	in VxWorks bootline
 *                      
 *     Required: -
 *     Switches: MENMON
 *
 *---------------------------[ Public Functions ]----------------------------
 *  MMPARA_SearchKey
 *	MMPARA_StringToInt
 *  MMPARA_StringToIp
 *-------------------------------[ History ]---------------------------------
 *
 * $Log: mmparastring.c,v $
 * cloned from common mmparastring 1.3
 *---------------------------------------------------------------------------
 * (c) Copyright 2002 by MEN mikro elektronik GmbH, Nuernberg, Germany 
 ****************************************************************************/
/* This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include "mmparastring.h"

#ifndef NULL
# define NULL (void *)0
#endif

static int strncmp2(char *s1, char *s2, int  n)
/*  register const char *s1, *s2; */
/*  register int n; */
{
/* Compare 2 strings. */

  while (1) {
    if( n==0 ) return 0;
    if (*s1 != *s2) return(*s1 - *s2);
    if (*s1 == 0 )return(0);
    n--;
    s1++;
    s2++;
  }
}

static char *EatBlanks( char *p )
{
	while( *p==' ' || *p=='\t' )
		p++;
	return p;
}

/********************************* MMPARA_SearchKey **************************
 *
 *  Description: Search for a parameter in bootline or menmon parameters
 *			   
 *			     Copies the string found into the supplied buffer.
 *				 dst must be large enough...	
 *---------------------------------------------------------------------------
 *  Input......: key				parameter name (e.g. "cbr=")
 *				 src				source string containing parameters
 *				 dst				destination string 
 *  Output.....: returns:			<dst> or NULL if parameter not found
 *  Globals....: -
 ****************************************************************************/
char *MMPARA_SearchKey( char *key, char *src, char *dst )
{
	int keylen;
	char *orgDst = dst;

	if( (src == NULL) || (dst == NULL) || (key == NULL) )
		return NULL;

	keylen = strlen(key);

	while( *src ){
		src = EatBlanks(src);	/* skip spaces */

		if( strncmp2( key, src, keylen ) == 0 ){
			/*--- found it ---*/

			src += keylen;	/* skip parameter */
			
			/*--- copy string until next blank or EOS ---*/
			while( *src && *src!=' ')
				*dst++ = *src++;
			*dst = '\0';
			return orgDst;
		}
		/* search for next blank */
		while( *src && *src!=' ')
			src++;		
	}
	return NULL;
}

/********************************* MMPARA_StringToInt ***********************
 *
 *  Description: Convert string to integer accordung to "base"
 *			   
 *  If an illegal character is found, conversion is aborted and the return
 *	value points to the unsuccessfully parsed character
 *  Will handle positive integers only
 *---------------------------------------------------------------------------
 *  Input......: base			radix (2,10,16)
 *				 expr		    string to parse
 *  Output.....: returns:		ptr to first unsuccessfully parsed char
 *				 *valP			value parsed
 *  Globals....: -
 ****************************************************************************/
char *MMPARA_StringToInt( int base, char *expr, unsigned long *valP)
{
	unsigned long tmp=0, new;

	while( *expr ){
		if( *expr >= '0' && *expr <= '9' )
			new = *expr - '0';
		else if( *expr >= 'a' && *expr <= 'f' )
			new = *expr - 'a' + 10;
		else if( *expr >= 'A' && *expr <= 'F' )
			new = *expr - 'A' + 10;
		else
			goto ex;
		
		if( new >= base )
			goto ex;

		tmp = (tmp*base) + new;
		expr++;
	}
ex:
	*valP = tmp;
	return expr;
}

/********************************* MMPARA_StringToIp *************************
 *
 *  Description: Converts string to IP address
 *			   
 *	String must be in dotted notation (e.g. 192.1.1.1) and may contain
 *	additional chars (e.g 192.1.1.1:ffffff00) which are not parsed		   
 *---------------------------------------------------------------------------
 *  Input......: str			string to parse
 *  Output.....: returns		ptr into string to next char after IP address
 *								or NULL if invalid
 *				 *ipP			converted IP address
 *  Globals....: -
 ****************************************************************************/
char *MMPARA_StringToIp( char *str, unsigned long *ipP )
{
	int num,i;
	unsigned long ip=0;

	for(i=0; i<4; i++){
		num = 0;

		while(1){
			if( *str >= '0' && *str <= '9' )
				num = num * 10 + (*str - '0');
			else {
				if( i== 3 ){
					break;
				}
				else {
					if( *str == '.' ){
						str++; 
						break;
					}
					else
						return NULL;
				}
			}
			str++;	
		}
		ip |= num << (8*(3-i));
	}
	*ipP = ip;
	return str;
}

