/***************************************************************************
                                 descript.h
                             -------------------
    project:  USB-device for Philips PDIUSBD12D

    begin                : Tue Dec 18 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  Dec 12, 2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include  "usbdev.h"

#define USB_DEVICE_DESCRIPTOR_TYPE                0x01
#define USB_CONFIGURATION_DESCRIPTOR_TYPE         0x02
#define USB_STRING_DESCRIPTOR_TYPE                0x03
#define USB_INTERFACE_DESCRIPTOR_TYPE             0x04
#define USB_ENDPOINT_DESCRIPTOR_TYPE              0x05
#define USB_POWER_DESCRIPTOR_TYPE                 0x06


/*Lengths of various descriptor segments*/
#define DEV_LENGTH                  18      /*length of device desc.  */
#define CFG_LENGTH                  9       /*length of cfg. desc. seg*/
#define INT_LENGTH                  9       /*length of int. desc. seg*/
#define HID_LENGTH                  9       /*length of HID  desc. seg*/
#define END_LENGTH                  7       /*length of end. desc. seg*/



/*SINGLE BULK INTERFACE*/

/*for multiple byte values, LSB goes first                            */
UINT8 DEV_DESC[] = {DEV_LENGTH,             /*length of this desc.    */
                         USB_DEVICE_DESCRIPTOR_TYPE,            /*DEVICE descriptor       */
                         USB_SPEC_ID_LO,
                         USB_SPEC_ID_HI,
                         0xff,              /*device class            */
                         0xff,              /*device subclass         */
                         0xff,              /*device protocol         */
                         EP0_PACKET_SIZE,   // 16, max packet size         */
                         USB_VENDOR_ID_LO,    // HBM's vendor id
                         USB_VENDOR_ID_HI,
                         USB_PRODUCT_ID_LO,   // HBM's product id
                         USB_PRODUCT_ID_HI,
                         USB_REVISION_ID_LO,  // HBM's revision id
                         USB_REVISION_ID_HI,
                         MFG_STR_OFS,       /*index of manuf. string  */
                         PID_STR_OFS,       /*index of prod.  string  */
                         UNR_STR_OFS,       /*index of ser. # string  */
                         0x01               /*number of configs.      */
                         };

UINT8 CFG_DESC[] = {CFG_LENGTH,             /*length of this desc.    */
                         USB_CONFIGURATION_DESCRIPTOR_TYPE,     /*CONFIGURATION descriptor*/
                         0x20,0x00,         /*total length returned   */
                         0x01,              /*number of interfaces    */
                         0x01,              /*number of this config   */
                         CFG_STR_OFS,       /*index of config. string */
                         0xC0,              // self powered //Ki: 0x80,              /*attr.: bus powered      */
                         1,                 // self powered //Ki: 50,                /*max power (20 mA)      */

// Interface Descriptor:
                         INT_LENGTH,        /*length of this desc.    */
                         USB_INTERFACE_DESCRIPTOR_TYPE,         /*INTERFACE descriptor    */
                         0x00,              /*interface number        */
                         0x00,              /*alternate setting       */
                         0x02,              /*# of (non 0) endpoints  */
                         0xff,              /*interface class         */
                         0xff,              /*interface subclass      */
                         0xff,              /*interface protocol      */
                         INT_STR_OFS,       /*index of intf. string   */

/*The WDM driver for this board references the endpoints by 'pipe'    */
/*number: 0, 1, 2 et al in order below:                               */
                       /*Pipe 0 (endpoint 5) -> TX_3                  */
                         END_LENGTH,        /*length of this desc.    */
                         USB_ENDPOINT_DESCRIPTOR_TYPE,          /*ENDPOINT descriptor     */
                         0x82,              /*address (IN)            */
                         0x02,              /*attributes  (BULK)      */
//                         0x01,    //@iso  isochron
                         EP2_TX_FIFO_SIZE,0x00,  /*max packet size (64)    */
                         1,   // Ki: 1,                 /*interval (ms)           */

// EP1_RXDescr:
                       /*Pipe 1 (endpoint 4) -> RX_3                  */
                         END_LENGTH,        /*length of this desc.    */
                         USB_ENDPOINT_DESCRIPTOR_TYPE,          /*ENDPOINT descriptor     */
                         0x02,              /*address (OUT)           */
                         0x02,              /*attributes  (BULK)      */
                         EP2_RX_FIFO_SIZE,0x00,  /*max packet size (64)    */
                         1    // Ki:  1        /*interval (ms)           */
                         };

