/***************************************************************************
                                  global.h
                             -------------------
    project:  USB-device for Philips PDIUSBD12D

    begin                : Wed Dec 5 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  Sep 01, 2003

    updates:  Sep 01, 2003 Wessler: Blocking read

 ***************************************************************************/







/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// ~


#ifndef __GLOBAL__
#define __GLOBAL__    

//#define DEBUG_ON

#define DRIVER_DATE   "12.03.2004"
#define DRIVER_VERSION  "1.2.0"

#include <linux/poll.h>   // poll/select

extern  UINT16  int_status;   // only needed for debugging purpose

#ifdef DEBUG_ON
  
  #define DEBUG_MAX   0       // 0: alles, >0: nur n Zeilen
  
//  extern  INT32 debug_count;
//  extern  UINT8 debug_trigger;

//  #define DTRIGGER_ON   debug_trigger = 1
//  #define DTRIGGER_OFF  debug_trigger = 0
//  #define DPRINT(format, a...)  { if (debug_trigger) { debug_count++; if ((DEBUG_MAX==0) || (debug_count<DEBUG_MAX)) { printk(format, ## a); } } }

  #define DTRIGGER_ON
  #define DTRIGGER_OFF
  #define DPRINT(format, a...)  { printk(format, ## a); }
#else

  #define DPRINT(format, a...)
  #define DTRIGGER_ON
  #define DTRIGGER_OFF

#endif


#define outportb(port, val)   outb(val, port)
#define inportb(port)         inb(port)


// structure definitions:

typedef struct _device_request
{
  UINT8 bmRequestType;
  UINT8 bRequest;
  UINT16  wValue;
  UINT16  wIndex;
  UINT16  wLength;
} DEVICE_REQUEST;


#define MAX_CONTROLDATA_SIZE  8

typedef struct _control_xfer
{
  DEVICE_REQUEST DeviceRequest;
  UINT16  wLength;
  UINT16  wCount;
  UINT8 * pData;
  UINT8 dataBuffer[MAX_CONTROLDATA_SIZE];
} CONTROL_XFER;


typedef struct _IO_REQUEST {
  UINT16  uAddressL;
  UINT8 bAddressH;
  UINT16  uSize;
  UINT8 bCommand;
} IO_REQUEST, *PIO_REQUEST;







// main.c:
extern void d12_init(boolean reset);
extern void after_d12_interrupt();


#define inBufMax    4096
extern  UINT8 inBuffer[inBufMax];         // Buffer for data received from the USB

#define outBufMax   4096
extern  UINT8 outBuffer[outBufMax];       // Buffer for data to be transmitted by USB

extern  UINT8   *pWriteAppl;              // Pointer to Buffer of data to be written by usb_write. Always points to the next free byte.
extern  UINT8   *pSendUSB;                // Pointer to Buffer of data to be read to transmit on USB. Always points to the next data byte that has to be transmitted.
extern  UINT8   *pReadAppl;               // Application read call gets data from inBuffer where this pointer points at.
extern  UINT8   *pGetUSB;                 // OUT packet interrupts write data from USB to inBuffer where this pointer points at.

extern  boolean interface_open;           // Flag to tell driver that open occured

extern  wait_queue_head_t read_waitqueue; // waitqueue for blocking user read call operation


extern  unsigned int usb_poll(struct file *filp, poll_table *wait);

extern  int usbdev_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data);


// isr.c:
extern  void  ep0_rxdone(void);
extern  void  ep0_txdone(void);
extern  void  ep1_rxdone(void);
extern  void  ep1_txdone(void);

extern  void  ep2_rxdone(void);
extern  void  ep2_txdone(void);
extern  void  d12_restart(void);

extern  void  delayc(UINT16 value);


extern  boolean flag_remote_wakeup;
extern  boolean flag_configuration;
extern  boolean flag_suspend;
extern  boolean flag_bus_reset;

extern  UINT8 control_state;

extern  CONTROL_XFER  ControlData;

#define UNILEN        50
extern  char  unistr[UNILEN+2];



// pdi_d12.c:
extern  UINT8 read_usbd(void);
extern  void  write_usbd(UINT8 data);
extern  UINT8 read_usbc1(UINT8 addr);
extern  UINT16  read_usbc2(UINT8 addr);
extern  void  write_usbc0(UINT8 addr);
extern  void  write_usbc1(UINT8 addr, UINT8 data);
extern  void  write_usbc2(UINT8 addr, UINT8 data1, UINT8 data2);

extern  UINT8 d12_ReadEndpoint(UINT8 endpx, UINT8 * buf, UINT32 len);
extern  UINT16 d12_ReadMainEndpoint(UINT8 * buf, UINT32 len);

extern  UINT8 d12_WriteEndpoint(UINT8 endpx, UINT8 * buf, UINT32 len);
extern  UINT8 d12_WriteMainEndpoint(UINT8 * buf, UINT32 len);
extern  void  d12_AcknowledgeEndpoint(UINT8 endpx);
extern  void  d12_SetEndpointStatus(UINT8 endpx, UINT8 stalled);

extern  void  d12_ClearEndpX(UINT8 EPIndex);



// request.c:
extern  void control_handler(void);

extern  void code_transmit(UINT8 /*code*/ * pRomData, INT16 len);
extern  void string_transmit(INT32 index);
extern  INT32 mak_uni_str(int INT32 index);

extern  void get_status(void);
extern  void clear_feature(void);
extern  void set_feature(void);
extern  void set_address(void);
extern  void get_descriptor(void);
extern  void get_configuration(void);
extern  void set_configuration(void);
extern  void get_interface(void);
extern  void set_interface(void);



extern  INT8  USB_SerialNo_Str[];
extern  UINT8 CFG_DATA[];
extern  UINT8 MFG_DATA[];


#endif  // __global__
