/***************************************************************************
                                    isr.c
                             -------------------
    project:  USB-device for Philips PDIUSBD12D

    begin                : Thu Dec 13 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  Sep 01, 2003

    updates:  May 09, 2003 Wessler: Check if Control In buffer is empty before writing
              Sep 01, 2003 Wessler: Blocking read

 ***************************************************************************/




/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/*  Um Verwirrungen zu vermeiden schreibe ich bei Variablen und Defines ein "X" vor der Endpoint-Nummer,
    wenn es sich dabei um einen Endpoint-Index handelt, und lasse den Index weg, wenn es der Endpoint selber ist.
    Die Zuordnung ist gem Philips-Nomenklatur:
      Endpoint Number   Endpoint Index  Transfer Type
            0                  0         Control Out
            0                  1         Control In
            1                  2         Generic Out
            1                  3         Generic In
            2                  4         Generic Out
            2                  5         Generic In
*/      


#include  <asm/io.h>
#include  <linux/interrupt.h>   // Tasklets

#include  <asm/dma.h>
#include  <asm/page.h>          // __va()
#include  <asm/irq.h>

#include  "types.h"
#include  "global.h"
#include  "pdi_d12.h"
#include  "usbdev.h"


boolean flag_remote_wakeup;
boolean flag_configuration;
boolean flag_suspend;
boolean flag_bus_reset;

UINT16  int_status = 0;
UINT8 control_state;


// Control endpoint TX/RX buffers
CONTROL_XFER  ControlData;

UINT8 GenEpBuf[EP1_PACKET_SIZE];    
INT8  unistr[UNILEN+2];


void after_d12_interrupt(void)
{

  DPRINT("t");

  int_status = read_usbc2(D12_READ_INT_REG);    // Read Interrupt Register, word size!
  int_status &= (D12_INT_BUSRESET | D12_INT_ENDP2IN | D12_INT_ENDP2OUT | D12_INT_SUSPENDCHANGE | D12_INT_ENDP0IN | D12_INT_ENDP0OUT | D12_INT_ENDP1IN | D12_INT_ENDP1OUT);

  while (int_status)
  {
    DPRINT("%02X ", int_status);

    if (int_status & D12_INT_BUSRESET)
    {
      flag_bus_reset = TRUE;        
    }
    else
    {
      flag_bus_reset = FALSE;

      if (int_status & D12_INT_ENDP2IN)
        ep2_txdone();
        
      if (int_status & D12_INT_ENDP2OUT)
        ep2_rxdone();
      
      if (int_status & D12_INT_SUSPENDCHANGE)
        flag_suspend = TRUE;
      else
        flag_suspend = FALSE;
        

      if (int_status & D12_INT_ENDP0IN)
        ep0_txdone();
        
      if (int_status & D12_INT_ENDP0OUT)
        ep0_rxdone();
        
      if (int_status & D12_INT_ENDP1IN)
        ep1_txdone();
        
      if (int_status & D12_INT_ENDP1OUT)
        ep1_rxdone();
    }

  int_status = read_usbc2(D12_READ_INT_REG);    // Read Interrupt Register, word size!
  int_status &= (D12_INT_ENDP2IN | D12_INT_ENDP2OUT | D12_INT_SUSPENDCHANGE | D12_INT_ENDP0IN | D12_INT_ENDP0OUT | D12_INT_ENDP1IN | D12_INT_ENDP1OUT);

  }

  DPRINT("e ");

  ENABLE_INTERRUPT;

}   // d12_interrupt

        

  
void  ep0_rxdone(void)
{
UINT8 ep_last,  b;


  ep_last = read_usbc1(D12_READ_LAST_TRANS_STATX +0);   // Read Last Transaction Status Register Endpx 0, byte size
  // Interrupt-Flag cleared
    
  DPRINT("ep0_rx: ep_last=%02X.\n", ep_last);

  if (ep_last & D12_SETUPPACKET)
  {
    ControlData.wLength = 0;
    ControlData.wCount = 0;

    b = d12_ReadEndpoint(0, (&ControlData.DeviceRequest.bmRequestType), sizeof(ControlData.DeviceRequest));

    if (b != sizeof(DEVICE_REQUEST))
    {   // error happened:
      write_usbc1(D12_SET_ENDPX_STATUS +0, D12_STALLED);
      write_usbc1(D12_SET_ENDPX_STATUS +1, D12_STALLED);
      control_state = USB_IDLE;
      return;
    }

    // Acknowledge setup here to unlock in/out endp
    write_usbc0(D12_SELECT_ENDPX0);     // Select Endpoint Index 0
    write_usbc0(D12_ACK_SETUP);         // Acknowledge Setup
    write_usbc0(D12_CLEAR_BUFFER);      // Clear Buffer
    write_usbc0(D12_SELECT_ENDPX1);     // Select Endpoint Index 1
    write_usbc0(D12_ACK_SETUP);         // Acknowledge Setup
    
    ControlData.wLength = ControlData.DeviceRequest.wLength;
    ControlData.wCount = 0;

    if (ControlData.DeviceRequest.bmRequestType & (UINT8)USB_ENDPOINT_DIRECTION_MASK)
    {
      control_state = USB_IDLE;   /* get command */
      control_handler();
    }
    else
    {
      if (ControlData.DeviceRequest.wLength == 0)
      {
        control_state = USB_IDLE;   /* set command */
        control_handler();
      }
      else
      {
        if(ControlData.DeviceRequest.wLength > MAX_CONTROLDATA_SIZE)
        {
          control_state = USB_IDLE;

          write_usbc1(D12_SET_ENDPX_STATUS +0, D12_STALLED);
          write_usbc1(D12_SET_ENDPX_STATUS +1, D12_STALLED);
        }
        else
        {
          control_state = USB_RECEIVE;  /* set command with OUT token */
        }
      } // set command with data
    } // else set command
  } // if setup packet

  else
  {
    if (control_state == USB_RECEIVE)
    {
      b = d12_ReadEndpoint(0, ControlData.dataBuffer +ControlData.wCount, EP0_PACKET_SIZE);
    
      ControlData.wCount += b;
      if( b != EP0_PACKET_SIZE || ControlData.wCount >= ControlData.wLength)
      {
        control_state = USB_IDLE;
        control_handler();
      }
    }
    else
    {
      control_state = USB_IDLE;
    }
  }
}   // ep0_rxdone



  
void  ep0_txdone(void)
{
UINT16  i;
UINT8 btemp;

  btemp = read_usbc1(D12_READ_LAST_TRANS_STATX +1);   // Read Last Transaction Status Register Endpx 1: Clear interrupt flag
  // Interrupt-Flag cleared

  DPRINT(" ep0_tx. ctrl=%i, rltsr=%02X - ", control_state, btemp);

  if (control_state != USB_TRANSMIT)
  {
    return;
  }

  if ( read_usbc1(D12_READ_EP_STATUS_X1) & (D12_STATUS_BUFFER0_FULL | D12_STATUS_BUFFER0_FULL) )
  { // Control In buffer is not empty: keep control_state=USB_TRANSMIT and do nothing:
    DPRINT("Ci");
    return;
  }

  i = ControlData.wLength - ControlData.wCount;

  DPRINT("ctrl==TRANSMIT, i=%d\n", i);

  if (i >= EP0_PACKET_SIZE)
  { // Ki: Some changes around here compared to Philips demo:
    // control_state stays USB_IDLE
    i = EP0_PACKET_SIZE;
  }
  else
  { //Ki: may be zero packet at the end
    control_state = USB_IDLE;
  }
  
  d12_WriteEndpoint(1, ControlData.pData + ControlData.wCount, i);
  ControlData.wCount += i;

}   // ep0_txdone



void  ep1_rxdone(void)
{
UINT8 len;
UINT8 btemp;

  btemp = read_usbc1(D12_READ_LAST_TRANS_STATX +2);   // Read Last Transaction Status Register Endpx 2: Clear interrupt flag

  len = d12_ReadEndpoint(2, GenEpBuf, sizeof(GenEpBuf));

  DPRINT("ep1_rx: RLTS2=%02Xh, ReadEnp2: len=%d\n", btemp, len);

}   // ep1_rxdone



void ep1_txdone(void)
{
UINT8 btemp;

  DPRINT("ep1_tx\n");

  btemp = read_usbc1(D12_READ_LAST_TRANS_STATX +3);   // Read Last Transaction Status Register Endpx 3: Clear interrupt flag

} //  ep1_txdone



UINT8   TxBuffer[EP2_TX_FIFO_SIZE];

void ep2_txdone(void)
{ // In Philips example called: main_txdone
UINT8   btemp;
INT32   ret_transmitted;
INT32   filled;
UINT8   *pWrite, *pSend;
UINT8   *pBuffer;
INT32   toCopy;

  btemp = read_usbc1(D12_READ_LAST_TRANS_STATX +5);   // Read Last Transaction Status Register Endpx 5: Clear interrupt flag

  DPRINT("ep2_tx\n");

//  filled = pWrite -pSend;

  pWrite = pWriteAppl;
  pSend = pSendUSB;

  if (pWrite == pSend)
    d12_WriteMainEndpoint(outBuffer, 0);
  else
  {

    if (pSend >= outBuffer +outBufMax)
    {
      if (pWrite > outBuffer)
        pSend = outBuffer;
      else
      {
        d12_WriteMainEndpoint(outBuffer, 0);
        return;
      }
    }


    filled = pWrite -pSend;

    if (filled < 0)
    { // pSend is right of pWrite:
      filled = (outBuffer -pSend) +outBufMax;

      if (filled >= EP2_TX_FIFO_SIZE)
      {
        ret_transmitted = d12_WriteMainEndpoint(pSend, EP2_TX_FIFO_SIZE);
        pSend += ret_transmitted;
      }
      else
      { // D12 fifo buffer is bigger than data right of pSend, get all data right of pSend...:
        pBuffer = &TxBuffer[0];
        memcpy(pBuffer, pSend, filled);
        toCopy = filled;
        pSend += filled;

        // ...and then try to get some from the left boundary of outBuffer:
        pBuffer += filled;
        filled = pWrite -outBuffer;
        if (toCopy +filled > EP2_TX_FIFO_SIZE)
          filled = EP2_TX_FIFO_SIZE -toCopy;
        memcpy(pBuffer, outBuffer, filled);
        toCopy += filled;
        pSend = outBuffer +filled;

        // finally try to transmit this buffer:
        ret_transmitted = d12_WriteMainEndpoint(&TxBuffer[0], toCopy);
      }
    }
    else
    {
      ret_transmitted = d12_WriteMainEndpoint(pSend, filled);
      pSend += ret_transmitted;
    }

  pSendUSB = pSend;

  }

  return;

} //  ep2_txdone



UINT8 RxBuffer[EP2_RX_FIFO_SIZE *2];

void ep2_rxdone(void)
{ // In Philips example called: main_rxdone
INT32 toCopy;
UINT8 btemp;
UINT8 *pBuffer;
UINT8 *pRead;
UINT8 *pGet;
INT32  free_right;



  btemp = read_usbc1(D12_READ_LAST_TRANS_STATX +4);   // Read Last Transaction Status Register Endpx 4: Clear interrupt flag
  
  DPRINT("ep2_rx: RLTS4=%02Xh\n", btemp);

  pRead = pReadAppl;
  pGet = pGetUSB;
  pBuffer = &RxBuffer[0];

  toCopy = d12_ReadMainEndpoint(pBuffer, EP2_RX_FIFO_SIZE *2);

  if (toCopy > 0)
  { // some data received from USB, now copy them into ring buffer:

    if (pGet >= inBuffer +inBufMax)
    { // pGetUSB at very right boundary:
      if (pRead > inBuffer +1)
      { // there is still some space left of pReadAppl, so set pGetUSB at the left boundary:
        pGet = inBuffer;
      }
      else
      {
        printk("%s:  BulkOUTBuffer full. Application should read this device!\n", MODULE_NAME);
        wake_up_interruptible(&read_waitqueue); // it's time that user read call gets some data!
        return; // inBuffer is totally filled. All USB OUT packets written to Buffer are lost.
      }
    }

    if (pGet >= pRead)
    { // PGetUSB is right of pReadAppl
      free_right = (inBuffer -pGet) +inBufMax;

      if (free_right >= toCopy)
      { // there's enough space right of pGetUSB, directly copy everything:

        memcpy(pGet, pBuffer, toCopy);
        pGet += toCopy;
      }
      else
      {
        memcpy(pGet, pBuffer, free_right);
        pGet += free_right;
        pBuffer += free_right;
        toCopy -= free_right;

        if (pRead > inBuffer +1)
        { // there is some space left of pReadAppl:
          pGet = inBuffer; // place pGet at very left boundary
          free_right = pRead -inBuffer -1;

          if (free_right < toCopy)
            toCopy = free_right;

          memcpy(pGet, pBuffer, toCopy);
          pGet += toCopy;
        }
        else
        {
          printk("%s:  BulkOUTBuffer full. Application should read this device!\n", MODULE_NAME);
          wake_up_interruptible(&read_waitqueue); // it's time that user read call gets some data!
          return;   // discard the whole last USB transmission;
        }
      }
    }
    else
    { // pGet < pRead
      free_right = (pRead -pGet) -1;

      if (free_right < toCopy)
      {
        wake_up_interruptible(&read_waitqueue); // it's time that user read call gets some data!
        return;   // discard the whole last USB transmission;
      }

      memcpy(pGet, pBuffer, toCopy);
      pGet += toCopy;
    }

  pGetUSB = pGet;
  wake_up_interruptible(&read_waitqueue);   // let the user read call get some data

  }

  return;
} //  ep2_rxdone



void d12_restart(void)
{
  d12_init(TRUE);
}



void delayc(UINT16 value)
{

UINT16  i;

if (value)

  for (i=0; i< value; i++)
    ;

} // delayc
