/***************************************************************************
                                  pdi_d12.h
                             -------------------
    project:  USB device for Philips PDIUSBD12D

    begin                : Wed Dec 5 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  May, 09, 2003
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __PDI_D12__
#define __PDI_D12__

// #include "../../cpxxapp/cp64/target.h"
#define CP22

#include  <asm/irq.h>
#include  <linux/sched.h>   // cli()




// Defines depending on hardware layout:
#ifdef CP42
  #define USB_IRQ_NO      6                     // IRQ no for USB chip
  #define DRIVER_TARGET   "CP42"
  #define portbase_addr   0x27E                 // base address to port of USB chip
  #define PDIADDR         0                     // Address line no. for USB Chip
  #define DMA_CHANNEL     1
#endif

#ifdef CP22
  #define USB_IRQ_NO      15                    // IRQ no for USB chip
  #define DRIVER_TARGET   "CP22"

  #define portbase_addr   0x278                 // base address to port of USB chip
  #define PDIADDR         2                     // Address line no. for USB Chip
  #define DMA_CHANNEL     1
#endif




// #######################################



#ifdef DEBUG_ON
  extern  INT32 disint;
  #define DISABLE_INTERRUPT   { disable_irq(USB_IRQ_NO); disint += 1; /*DPRINT("disint=%d\n", disint);*/ }
  #define ENABLE_INTERRUPT    { enable_irq(USB_IRQ_NO); disint -=1; /*DPRINT("disint=%d\n", disint);*/ }
#else
  #define DISABLE_INTERRUPT   { disable_irq_nosync(USB_IRQ_NO); }
  #define ENABLE_INTERRUPT    { enable_irq(USB_IRQ_NO); }
#endif


// Set USB_IRQ_NO to level sensitive, since it is not by default, I found out reading 0x4D1!
// See ZFx86 data book 1.0 Rev B, Table 4.45, works just the same on ETX board for CP42.
// Check Table 4.52 for interrupt polarity too!
#define SET_INTERRUPT_LEVEL outb( (inb((USB_IRQ_NO > 7) ? 0x4D1 : 0x4D0) | (0x01 << (USB_IRQ_NO %8))), (USB_IRQ_NO > 7) ? 0x4D1 : 0x4D0)  

#define portbase_len    (2* (1 << PDIADDR))   // bytes of port needed

#define D12_COMMAND     (portbase_addr +(1 << PDIADDR)) // command address to USB chip
#define D12_DATA        (portbase_addr +0)              // data address to USB chip




// Defines depending on USB chip:
#define CLOCKDIV        8

#define MAX_ENDPOINTS       (INT8)0x03


#define EP0_TX_FIFO_SIZE   16
#define EP0_RX_FIFO_SIZE   16
#define EP0_PACKET_SIZE    16

#define EP1_TX_FIFO_SIZE   16
#define EP1_RX_FIFO_SIZE   16
#define EP1_PACKET_SIZE    16

#define EP2_TX_FIFO_SIZE   64
#define EP2_RX_FIFO_SIZE   64
#define EP2_PACKET_SIZE    64



// DMA registers in CPU chipset:
#define DMA1REG_CHANNEL(c)  (2*(c))    // address of register for channel c

#define DMA1REG_COMMAND     (0x008)    // address of command register
#define DMA1REG_SMASK       (0x00A)    // address of single mask register
#define DMA1REG_MODE        (0x00B)    // address of mode register
#define DMA1REG_FFC         (0x00C)    // adress clear first/last flip flop
#define DMA1REG_MASTER_CLR  (0x00D)    // master clear


// Bits and values in DMA registers:
#define DMA37MD_SINGLE      0x40
#define DMA37MD_BLOCK       0x80
#define DMA37MD_AUTO        0x10
#define DMA37MD_DEMAND      0x00
#define DMA37MD_WRITE       0x04      // write USB chip, read memory operation
#define DMA37MD_READ        0x08      // read USB chip, write memory operation



// D12 command addresses:
#define D12_SET_ADDRESS_ENABLE        0xD0    // Set Address / Enable
#define D12_SET_ENDPOINT_ENABLE       0xD8    // Set Endpoint Enable
#define D12_SET_MODE                  0xF3    // Set Mode
#define D12_SET_DMA                   0xFB    // Set DMA
#define D12_READ_INT_REG              0xF4    // Read Interrupt Register
#define D12_SELECT_ENDPX              0x00    // Select Endpoint (base address)
#define D12_SELECT_ENDPX0             0x00    // Select Endpoint Control Out
#define D12_SELECT_ENDPX1             0x01    // Select Endpoint Control In
#define D12_SELECT_ENDPX2             0x02    // Select Endpoint 1 Out
#define D12_SELECT_ENDPX3             0x03    // Select Endpoint 1 In
#define D12_SELECT_ENDPX4             0x04    // Select Endpoint 2 Out
#define D12_SELECT_ENDPX5             0x05    // Select Endpoint 2 In
#define D12_READ_LAST_TRANS_STATX     0x40    // Read Last Transaction Register

#define D12_READ_BUFFER               0xF0    // Read Buffer
#define D12_WRITE_BUFFER              0xF0    // Write Buffer
#define D12_SET_ENDPX_STATUS          0x40    // Set Endpoint Status (base address)
#define D12_ACK_SETUP                 0xF1    // Acknowledge Setup
#define D12_CLEAR_BUFFER              0xF2    // Clear Buffer
#define D12_VALIDATE_BUFFER           0xFA    // Validate Buffer

#define D12_SEND_RESUME               0xF6    // Send Resume

#define D12_READ_EP_STATUS_X0         0x80    // Read Endpoint Status, Endpoint Control Out
#define D12_READ_EP_STATUS_X1         0x81    // Read Endpoint Status, Endpoint Control In
  #define D12_STATUS_BUFFER0_FULL       0x20  //   Bit: Buffer 0 full
  #define D12_STATUS_BUFFER1_FULL       0x40  //   Bit: Buffer 1 full



// Bits in Set Address / Enable:
#define D12_ADDRESS_ENABLE            0x80


// Bits in Set Mode:
#define D12_NO_LAZYCLOCK              0x02
#define D12_CLOCK_RUNNING             0x04
#define D12_INTERRUPT_MODE            0x08
#define D12_SOFTCONNECT               0x10

#define D12_SET_TO_ONE                0x40


// Values in Set DMA:
#define D12_BURST1                    0x00
#define D12_BURST4                    0x01
#define D12_BURST8                    0x02
#define D12_BURST16                   0x03


// Bits in Set DMA:
#define D12_DMA_ENABLE                0x04
#define D12_DMA_DIRECTION_READ        0x00
#define D12_DMA_DIRECTION_WRITE       0x08
#define D12_DMA_AUTO_RELOAD           0x10
#define D12_INTERRUPT_PIN_MODE        0x20
#define D12_ENDPX4_INT_ENABLE         0x40
#define D12_ENDPX5_INT_ENABLE         0x80


// Bits in Last Transaction Status Register:
#define D12_SETUPPACKET               0x20


// Bits in Select Endpoint:
#define D12_FULLEMPTY                 0x01    // Full/Empty
#define D12_STALL                     0x02    // Stall


// Bits in Select Endpoint Status:

#define D12_STALLED                   0x01    // Stalled


// Bits in Read Interrupt Register:
#define D12_INT_ENDP0OUT              0x0001
#define D12_INT_ENDP0IN               0x0002
#define D12_INT_ENDP1OUT              0x0004
#define D12_INT_ENDP1IN               0x0008
#define D12_INT_ENDP2OUT              0x0010
#define D12_INT_ENDP2IN               0x0020
#define D12_INT_BUSRESET              0x0040
#define D12_INT_SUSPENDCHANGE         0x0080

#define D12_INT_DMA_EOT               0x0100


// Bits in Address:
#define USB_ENDPOINT_DIRECTION_MASK   0x80    // Endpoint direction bit,


#define USB_IDLE            0
#define USB_TRANSMIT        1
#define USB_RECEIVE         2


#endif
