/***************************************************************************
                                  request.c
                             -------------------
    project:  USB device for Philips PDIUSBD12D

    begin                : Mon Dec 17 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  Dec 12, 2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include  <asm/io.h>      // DISABLE_INTERRUPT, ENABLE_INTERRUPT

#include  "types.h"
#include  "request.h"
#include  "global.h"
#include  "pdi_d12.h"
#include  "strings.h"
#include  "descript.h"
#include  <asm/irq.h>


#ifdef  DEBUG_ON
  INT32 debug_count = 0;      // counter for debugging feature in control_handler()
  boolean debug_trigger = FALSE;  
#endif


void stall_ep0(void)
{

  write_usbc1(D12_SET_ENDPX_STATUS +0, D12_STALLED);       // set status : stall 0
  write_usbc1(D12_SET_ENDPX_STATUS +1, D12_STALLED);       // set status : stall 1

  DPRINT("stall_ep0\n");

}

// **********************************************

void get_status(void)
{
  UINT8 endp, txdat[2];
  UINT8 bRecipient;
  UINT8 c1;

  bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;
  txdat[0]=0;
  txdat[1]=0;
  if (bRecipient == USB_RECIPIENT_DEVICE)
  {
    DPRINT("get_status: USB_RECIPIENT_DEVICE, flag_remote_wakeup=%d\n", flag_remote_wakeup);
    if (flag_remote_wakeup) 
      txdat[0] = 2;   // send 2 0
    d12_WriteEndpoint(1, txdat, 2);
  }
  else
  {
    if (bRecipient == USB_RECIPIENT_INTERFACE)
    {
      DPRINT("get_status: USB_RECIPIENT_INTERFACE\n");
      d12_WriteEndpoint(1, txdat, 2);          // send 0 0
    }
    else
    {
      if (bRecipient == USB_RECIPIENT_ENDPOINT)
      {
        DPRINT("get_status: USB_RECIPIENT_ENDPOINT\n");
        endp = (ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS) << 1;
        if (ControlData.DeviceRequest.wIndex & USB_ENDPOINT_DIRECTION_MASK)
          ++endp; /* Control-in */
        c1 = read_usbc1(endp);    /* Control-out or Control-in*/
        if(c1 & D12_STALL)  
          txdat[0] = 1;
        d12_WriteEndpoint(1, txdat, 2);
      }
      else
      {
        stall_ep0();
      }
    }
  }
  
}   // get_status



void clear_feature(void)
{
UINT8 endp;
UINT8 bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;

  DPRINT("  clear_feature.\n");
    
  if (bRecipient == USB_RECIPIENT_DEVICE
    && ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP)
  {
    flag_remote_wakeup = FALSE;
    d12_WriteEndpoint(1, 0, 0);
  }
  else
  {
    if (bRecipient == USB_RECIPIENT_ENDPOINT
      && ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL)
    {
      endp = (UINT8)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
      if (ControlData.DeviceRequest.wIndex & USB_ENDPOINT_DIRECTION_MASK)
      {
        write_usbc1((endp*2) | + 0x41, 0);
      }
      else
      {
        write_usbc1((endp*2) + 0x40, 0);
      }
      d12_WriteEndpoint(1, 0, 0);
    }
    else
    {
      stall_ep0();
    }
  }

}   // clear_feature


void set_feature(void)
{
  UINT8 endp;
  UINT8 bRecipient = ControlData.DeviceRequest.bmRequestType & USB_RECIPIENT;
  
  DPRINT("  set_feature.\n");
  
  if (bRecipient == USB_RECIPIENT_DEVICE
    && ControlData.DeviceRequest.wValue == USB_FEATURE_REMOTE_WAKEUP)
  {
    flag_remote_wakeup = TRUE;
    d12_WriteEndpoint(1, 0, 0);
  }
  else
  {
    if (bRecipient == USB_RECIPIENT_ENDPOINT
      && ControlData.DeviceRequest.wValue == USB_FEATURE_ENDPOINT_STALL)
    {
      endp = (UINT8)(ControlData.DeviceRequest.wIndex & MAX_ENDPOINTS);
      if (ControlData.DeviceRequest.wIndex & USB_ENDPOINT_DIRECTION_MASK)
      {
        write_usbc1((endp*2) + 0x41, 1);
      }
      else
      {

        write_usbc1((endp*2) + 0x40, 1);
      }
      d12_WriteEndpoint(1, 0, 0);
    }
    else
    {
      stall_ep0();
    }
  }

}   // set_feature


void set_address(void)
{
UINT8 bAddress;

  bAddress = (ControlData.DeviceRequest.wValue & DEVICE_ADDRESS_MASK) | D12_ADDRESS_ENABLE;
  write_usbc1(D12_SET_ADDRESS_ENABLE, bAddress);
  d12_WriteEndpoint(1, 0, 0);

  DPRINT("  set_address: bAddress=%02X\n", bAddress);

}   // set_address


void get_descriptor(void)
{
  UINT8 bDescriptor = ControlData.DeviceRequest.wValue >> 8;

  DPRINT("  get_descriptor: ");
  
  if (bDescriptor == USB_DEVICE_DESCRIPTOR_TYPE)
  {
    DPRINT("DEVICE_DESCR\n");
    code_transmit(DEV_DESC, sizeof(DEV_DESC));
  }
  else
  {
    if (bDescriptor == USB_CONFIGURATION_DESCRIPTOR_TYPE)
    {
      DPRINT("CONFIG_DESCR\n");
      code_transmit(CFG_DESC, sizeof(CFG_DESC));
    }
    else
    {
      if (bDescriptor == USB_STRING_DESCRIPTOR_TYPE)
      {
        DTRIGGER_ON;
        DPRINT("STRING_DESCR\n");

        bDescriptor = ControlData.DeviceRequest.wValue & 0x00FF;
        string_transmit(bDescriptor);
      }
      else
      {
        DPRINT("stall_ep0\n");
        stall_ep0();
      }
    }
  }

}   // get_descriptor


void get_configuration(void)
{
UINT8 btemp;

  if (flag_configuration)
    btemp = 1;
  else
    btemp = 0;

  DPRINT("  get_configuration\n");
  
  d12_WriteEndpoint(1, &btemp, 1);
  
}   // get_configuration


void set_configuration(void)
{

  DPRINT("  set_configuration\n");
  
  if (ControlData.DeviceRequest.wValue == 0)
  {
    /* put device in unconfigured state */
    d12_WriteEndpoint(1, 0, 0);
    write_usbc1(D12_SET_ENDPOINT_ENABLE, 0);    /* Disable all endpoints but EPP0. */
    flag_configuration = FALSE;
  }
  else
  {
    if (ControlData.DeviceRequest.wValue == 1)
    {
      /* Configure device */
      d12_WriteEndpoint(1, 0, 0);

      write_usbc1(D12_SET_ENDPOINT_ENABLE, 0);    /* Disable all endpoints but EPP0. */
      write_usbc1(D12_SET_ENDPOINT_ENABLE, 1);    /* Enable  generic/iso endpoints. */

      flag_configuration = TRUE;
    }
    else
    {
      stall_ep0();
    }
  }
  
}   // set_configuration


void get_interface(void)
{
  UINT8 txdat = 0;        /* Only/Current interface = 0 */
  
  DPRINT("  get_interface\n");
  
  d12_WriteEndpoint(1, &txdat, 1);

}   // get_interface


void set_interface(void)
{
  
  DPRINT("  set_interface\n");
  
  if ((ControlData.DeviceRequest.wValue == 0) && (ControlData.DeviceRequest.wIndex == 0))
    {
    d12_WriteEndpoint(1, 0, 0);
    }
  else
    {
    stall_ep0();
    }

}   // set_interface



void string_transmit(INT32 index)
{
UINT8 * chap;

  if ( index < UNR_STR_OFS )
  {
    chap = STR_ind[index];
  }
  else
  {
    chap = unistr;
    if ( mak_uni_str(index) == 0 )
      mak_uni_str(SNR_STR_OFS);
  }
  code_transmit(chap,*chap);

} //  string_transmit



INT32 mak_uni_str(INT32 index)
{
INT32 i1;


INT32 i2;
INT8 * chap;
INT8 * fchap;
INT8 * fchap1;

  chap = &unistr[UNILEN+2];
  for ( i1 = UNILEN+2; i1--; )
    *(--chap) = 0;
    
  if ( index == UNR_STR_OFS )
    fchap = String_un;
  else
    fchap = USB_SerialNo_Str;

  i1 = 0;
  fchap1 = fchap;
  while ( *fchap1++ > 0 && i1 < UNILEN/2 )
    ++i1;

  if ( i1 >= UNILEN/2 )
    i1 = 0;

  i2 = i1;
  chap = unistr;
  *chap++ = (i1 + 1)<<1;  // len in bytes inc header
  *chap++ = 3;            // string data type

  while ( i1-- )
  {
    *chap++ = *fchap++;
    *chap++ = 0;
  }
  
  return i2;
} //  mak_uni_str



void code_transmit(UINT8 * pRomData, INT16 len)
{

  DPRINT("code_transmit:%i", len);

  ControlData.wCount = 0;
  if(ControlData.wLength > len)
    ControlData.wLength = len;

  ControlData.pData = pRomData;
  if( ControlData.wLength >= EP0_PACKET_SIZE)
  {
    d12_WriteEndpoint(1, ControlData.pData, EP0_PACKET_SIZE);
    ControlData.wCount += EP0_PACKET_SIZE;


    control_state = USB_TRANSMIT;
  }
  else
  {
    d12_WriteEndpoint(1, pRomData, ControlData.wLength);
    ControlData.wCount += ControlData.wLength;

    control_state = USB_IDLE;
  }
  
}   // code_transmit



//  ************************************************************


void control_handler(void)
{
UINT8 type, req;
  
  type = ControlData.DeviceRequest.bmRequestType & USB_REQUEST_TYPE_MASK;
  req = ControlData.DeviceRequest.bRequest & USB_REQUEST_MASK;

  DPRINT("* control_handler [%d]: type=%02Xh, req=%02Xh\n", debug_count, type, req);

  if (type == USB_STANDARD_REQUEST)
  {
    switch (req)
    {
      case GET_STATUS:        get_status();
                              break;

      case CLEAR_FEATURE:     clear_feature();
                              break;

      case SET_FEATURE:       set_feature();
                              break;

      case SET_ADDRESS:       set_address();
                              break;

      case GET_DESCRIPTOR:    get_descriptor();
                              break;

      case GET_CONFIGURATION: get_configuration();
                              break;

      case SET_CONFIGURATION: set_configuration();
                              break;

      case GET_INTERFACE:     get_interface();
                              break;

      case SET_INTERFACE:     set_interface();
                              break;

      default:                stall_ep0();// unknown, stall it
                              break;
    }
  }
  else
  {
/*    if (type == USB_VENDOR_REQUEST)
    {
      stall_ep0();
    }
    else
*/    
    {
      stall_ep0();
    }
  }
  
}   // control_handler

