/***************************************************************************
                                  global.h
                             -------------------
    project:  USB-device for Philips ISP1181A

    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// ~


#ifndef __GLOBAL__
#define __GLOBAL__

// #define DEBUG_ON
// #define PRINTD

#define DRIVER_DATE   "15.04.2010"
#define DRIVER_VERSION  "0.1.20"     // x.y.z - official versions: z even, inofficial: z odd


#include  <linux/poll.h>
#include  "isp1181.h"


#ifdef PRINTD
  #define printd1(format, a...)  { printk(format, ## a); }
#else
  #define printd1(format, a...)  { ; }
#endif
#define printd(format, a...)  { ; }

#ifdef DEBUG_ON
  #define DEBUG_MAX   0       // 0: alles, >0: nur n Zeilen
  extern  INT32 debug_count;
  extern  UINT8 debug_trigger;
  #define DTRIGGER_ON   debug_trigger = 1
  #define DTRIGGER_OFF  debug_trigger = 0
  #define DPRINT(format, a...)  { if (debug_trigger) { debug_count++; if ((DEBUG_MAX==0) || (debug_count<DEBUG_MAX)) { printk(format, ## a); } } }
#else
  #define DPRINT(format, a...)  { ; }
  #define DTRIGGER_ON
  #define DTRIGGER_OFF
#endif


#define outportb(port, val)   outb(val, port)
#define inportb(port)         inb(port)
#define outportw(port, val)   outw(val, port)
#define inportw(port)         inw(port)
#define outportl(port, val)   outl(val, port)
#define inportl(port)         inl(port)

#define swap(x)               ( ( ((x) & 0xff) << 8) | ( ((x) >> 8) & 0xff) )




// main.c:

#define INBufMax    4096
#define OUTBufMax   4096
#define IsoBufMax   4096

#define FLAG_INTERFACE_OPEN   0x01

typedef struct _tDevice
{
  UINT16  minor_no;
  struct  semaphore sem;            // locks this structure
  UINT8   flags;
  UINT8   index;
} tDevice;

extern  tDevice main_dev;
extern  tDevice iso1_dev;
extern  tDevice iso2_dev;
extern  tDevice iso3_dev;


typedef struct  _tBulkBuffer
{
  UINT8   OUTBuffer[OUTBufMax];
  UINT8   INBuffer[INBufMax];
  UINT8   *pWriteAppl;    // Pointer to outBuffer that write data from application into outBuffer.
  UINT8   *pSendUSB;      // Pointer to outBuffer that gets data out of outBuffer into USB chip, running after pWriteAppl.
  UINT8   *pGetUSB;       // Pointer to inBuffer that writes data from USB chip into inBuffer.
  UINT8   *pReadAppl;     // Pointer to inBuffer that gets data out of inBuffer to application, running after pGetUSB.
  UINT8   TempReadBuffer[OUTBufMax];
  UINT8   TempINBuffer[BULK_MAX_PACKET_SIZE *2];
  UINT8   TempOUTBuffer[BULK_MAX_PACKET_SIZE *2];
  UINT8   EndpointNoIN;
  UINT8   EndpointNoOUT;
  boolean FullblockWritten;
  UINT16  FifoSizeIN;
  UINT16  FifoSizeOUT;    // BULK_MAX_PACKET_SIZE
  wait_queue_head_t waitqueue;  // waitqueue for blocking user read call operation
} tBulkBuffer;

extern  tBulkBuffer MAINBuffer;


typedef struct  _tIsoBuffer
{
  UINT8   Buffer[IsoBufMax];
  INT32   ToSend;
  UINT8   Status;
  UINT8   Index;
  UINT32  Offset;
} tIsoBuffer;

extern  tIsoBuffer IsoBufferA[3];       // Double buffers either to get data within Write call
extern  tIsoBuffer IsoBufferB[3];       //   or feed ISP fifo within IN packet interrupt.
extern  tIsoBuffer *WrIsoBuffer[3];
extern  tIsoBuffer *RdIsoBuffer[3];

extern  UINT8   AlternateSetting;
extern  boolean EnableBulkDoubleBuffer;

extern  boolean EnableSOFInt;

extern  int usbdev_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data);



// buffer.c:
extern  UINT16  calc_buffersizes(UINT16 setfifo, UINT8 minor_no);

extern  void  InitMAINBuffer(void);

extern  INT32 ApplWriteIsoBuffer(tDevice *dev, const char *userBuffer, INT32 userCount);
extern  INT32 ApplWriteBulkBuffer(tBulkBuffer *BulkBuffer, const char *userBuffer, INT32 userCount);
extern  INT32 ApplReadBulkBuffer(tBulkBuffer *BulkBuffer, char *userBuffer, INT32 userCount);

extern  void  FeedBulkINFifo(tBulkBuffer *BulkData);
extern  void  GetBulkOUTFifo(tBulkBuffer *BulkData, UINT16 status);
extern  void  FeedIsoFifo(UINT8 i);

extern  spinlock_t Lock_FeedBulkIN;   // this is global, so FeedBulkINFifo is always called only once.
extern  spinlock_t Lock_FeedIso;      // this is global, so FeedIsoFifo is always called only once.

// isr.c:
extern  void  after_isp_interrupt();

extern  void  interrupt_CTRL_IN(void);
extern  void  interrupt_CTRL_OUT(void);
extern  void  interrupt_MAIN_IN(void);
extern  void  interrupt_MAIN_OUT(void);
extern  void  interrupt_ISO1(void);
extern  void  interrupt_ISO2(void);
extern  void  interrupt_ISO3(void);

extern  void  isp_restart(void);

extern  UINT32  delayc(UINT32 value);

extern  UINT32  int_status;           // true interrupt register read from isp
extern  UINT32  tasklet_jobs;         // jobs from outside tasklet, to be processed inside tasklet; we use the same bit structure as isp interrupt register
extern  boolean flag_remote_wakeup;
extern  boolean flag_configuration;
extern  boolean flag_suspend;
extern  boolean flag_bus_reset;

extern  UINT8 control_state;

// isp1181.c:
extern  UINT8 usb_rd1b(void);
extern  UINT16 usb_rd1w(void);
extern  void  usb_wr1b(UINT8 data);
extern  void  usb_wr1w(UINT16 data);
extern  UINT8 usb_wrc_rd1b(UINT8 addr);
extern  UINT16 usb_wrc_rd1w(UINT8 addr);
extern  UINT16 usb_wrc_rd2b(UINT8 addr);
extern  UINT32 usb_wrc_rd2w(UINT8 addr);
extern  UINT32 usb_wrc_rd1l(UINT8 addr);
extern  void usb_wrc(UINT8 addr);
extern  void usb_wrc_wr1b(UINT8 addr, UINT8 data);
extern  void usb_wrc_wr1w(UINT8 addr, UINT16 data);
extern  void usb_wrc_wr2w(UINT8 addr, UINT16 data1, UINT16 data2);
extern  void usb_wrc_wr1l(UINT8 addr, UINT32 data);

extern  void  calc_endpoints(void);
extern  void  isp_init(boolean reset);
extern  void  isp_SetAllMainEndpointStatus(UINT8 status);
extern  UINT16 isp_GetIsoFifoSize(UINT16 buffersize, UINT8 *fifo);
extern  UINT16 isp_GetIsoFreeFifoSize(UINT16 buffersize);
extern  UINT16 isp_GetNonIsoFifoSize(UINT16 buffersize, UINT8 *fifo);

extern  UINT16 isp_GetCtrlEndpoint(UINT8 *buf, UINT16 max_bytes);
extern  UINT16 isp_GetBulkEndpoint(tBulkBuffer *BulkData, UINT8 *buf, UINT16 max_bytes);
extern  UINT16 isp_WriteCtrlEndpoint(UINT8 *buf, UINT16 bytes_to_write);
extern  UINT16 isp_WriteBulkEndpoint(tBulkBuffer *BulkData, UINT8 *buf, UINT16 bytes_to_write);
extern  UINT16 isp_WriteIsoEndpoint(UINT8 i, UINT8 *buffer, INT16 bytes_to_write);

extern  boolean isp_isobuffer_empty(UINT16 status, UINT8 endp);


// descript.c:
extern  void control_handler(void);

extern  void ctrl_transmit(UINT8 *pbuf, INT16 len);
extern  void transmit_complete_descriptor(void);

extern  void get_status(void);
extern  void clear_feature(void);
extern  void set_feature(void);
extern  void set_address(void);
extern  void get_descriptor(void);

extern  void get_configuration(void);
extern  void set_configuration(void);
extern  void get_interface(void);
extern  void set_interface(void);
extern  void isp_StallCtrlEndpointWrite(void);
extern  void isp_StallCtrlEndpointRead(void);
extern  void StallCtrlEndpoint(void);

extern  void  copy_serialstr_to_serialdescr(void);

extern  INT8  USB_SerialNo_Str[];

extern  UINT8   num_active_endpoints;         // number of active endpoint, control endpoints not counted
extern  UINT16  config_descriptor_length;


#endif  // __global__

