/***************************************************************************
                                    isr.c
                              -------------------
    project:  USB-device for Philips ISP1181A

    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include  <linux/config.h>
#include  <linux/module.h>
#include  <asm/io.h>
#include  <linux/interrupt.h>   // Tasklets
#include  <linux/nmi.h>

#include  "types.h"
#include  "global.h"
#include  "isp1181.h"
#include  "descript.h"


boolean flag_remote_wakeup;
boolean flag_in_isr;
boolean flag_configuration;
boolean flag_suspend;
boolean flag_bus_reset;

UINT32  int_status = 0;
UINT8   control_state = 0;
UINT32  tasklet_jobs = 0;

// Control endpoint TX/RX buffers
CONTROL_XFER  ControlData;


void after_isp_interrupt(void)
{
UINT8   loop_count;

#ifdef CONFIG_OWN_NMI_HANDLER
extern INT16  usb_frame_counter;  // Kernel-Symbol nur fr Siegfried, h�
                                  // Jo! Alles nur fr mich. Nett, gelle?
#endif

  int_status = usb_wrc_rd1l(ISP_RD_INT_REG);    // Read Interrupt Register, long size!

  int_status &= (ISP_INT_BUSTATUS | ISP_INT_RESET | ISP_INT_ISO1 | ISP_INT_ISO2 | ISP_INT_ISO3
                  | ISP_INT_CTRL_IN | ISP_INT_CTRL_OUT | ISP_INT_MAIN_IN | ISP_INT_MAIN_OUT
                  | ISP_INT_SUSPND | ISP_INT_SOF);

//  if (int_status & ~ISP_INT_SOF)    //@@
//    printd("t%04X.%04X ", (UINT16)(int_status >> 16), (UINT16)(int_status & 0xFFFF));

  loop_count = 100;

  while ( (int_status & ~(ISP_INT_BUSTATUS)) && (loop_count--))
  {

    if (int_status & ISP_INT_RESET)
    {
      flag_bus_reset = TRUE;
      DPRINT("%s: USB reset\n", MODULE_NAME);
      delayc(10000);
      isp_restart();
    }
    else
      flag_bus_reset = FALSE;

#ifdef CONFIG_OWN_NMI_HANDLER      
    if (int_status & ISP_INT_SOF)
    {
      usb_frame_counter = usb_wrc_rd1w(ISP_READ_FRAME_NUMBER);  // Get last SOF number
      printd(".%.04d", lastSOFNo);
    }
#endif    

    if (int_status & ISP_INT_ISO1)
      interrupt_ISO1();

    if (int_status & ISP_INT_ISO2)
      interrupt_ISO2();

    if (int_status & ISP_INT_ISO3)
      interrupt_ISO3();

    if (int_status & ISP_INT_MAIN_IN)
      interrupt_MAIN_IN();

    if (int_status & ISP_INT_MAIN_OUT)
      interrupt_MAIN_OUT();

    if (int_status & ISP_INT_CTRL_IN)
      interrupt_CTRL_IN();

    if (int_status & ISP_INT_CTRL_OUT)
      interrupt_CTRL_OUT();

    if (tasklet_jobs)
    {
      if (tasklet_jobs & ISP_INT_ISO1)
      {
        tasklet_jobs &= ~(ISP_INT_ISO1);
        FeedIsoFifo(0);
      }

      if (tasklet_jobs & ISP_INT_ISO2)
      {
        tasklet_jobs &= ~(ISP_INT_ISO2);
        FeedIsoFifo(1);
      }

      if (tasklet_jobs & ISP_INT_ISO3)
      {
        tasklet_jobs &= ~(ISP_INT_ISO3);
        FeedIsoFifo(2);
      }

      if (tasklet_jobs & ISP_INT_MAIN_IN)
      {
        tasklet_jobs &= ~(ISP_INT_MAIN_IN);
        FeedBulkINFifo(&MAINBuffer);
      }
    }

    if (int_status & ISP_INT_SUSPND)
      flag_suspend = TRUE;
    else
      flag_suspend = FALSE;

    int_status = usb_wrc_rd1l(ISP_RD_INT_REG);    // Read Interrupt Register, long size!

    int_status &= (ISP_INT_BUSTATUS | ISP_INT_RESET | ISP_INT_ISO1 | ISP_INT_ISO2 | ISP_INT_ISO3
                    | ISP_INT_CTRL_IN | ISP_INT_CTRL_OUT | ISP_INT_MAIN_IN | ISP_INT_MAIN_OUT
                    | ISP_INT_SUSPND | ISP_INT_SOF);

  }

  printd("e ");

  ENABLE_INTERRUPT;

}   // after_isp_interrupt



void  interrupt_CTRL_OUT(void)
{   // Receive data from control endpoint
UINT8   ep_status;
UINT16  wtemp;


  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_CTRL_OUT);   // clears interrupt bit


  printd("CtrlOUT: ep_status=%02Xh.\n", ep_status);

  if (ep_status & ISP_EP_STATUS_SETUPT)
  {
    ControlData.wLength = 0;
    ControlData.wCount = 0;

    wtemp = isp_GetCtrlEndpoint(&ControlData.DeviceRequest.bmRequestType, sizeof(ControlData.DeviceRequest));

    if (wtemp != sizeof(DEVICE_REQUEST))
    {   // error happened:
      StallCtrlEndpoint();
      control_state = USB_IDLE;

      return;
    }

    // Acknowledge setup here to unlock in/out endp:
    usb_wrc(ISP_ACK_SETUP);
    usb_wrc(ISP_CLEAR_BUFFER_CTRL_OUT);

    ControlData.wLength = ControlData.DeviceRequest.wLength;
    ControlData.wCount = 0;

    if (ControlData.DeviceRequest.bmRequestType & (UINT8)USB_ENDPOINT_DIRECTION_MASK)
    {
      control_state = USB_IDLE;   // get command
      control_handler();
    }
    else
    {
      if (ControlData.DeviceRequest.wLength == 0)
      {
        control_state = USB_IDLE;   // set command
        control_handler();
      }
      else
      {
        if(ControlData.DeviceRequest.wLength > MAX_CONTROLDATA_SIZE)
        {
          control_state = USB_IDLE;
          usb_wrc(ISP_STALL_CTRL_OUT);
          usb_wrc(ISP_STALL_CTRL_IN);
        }
        else
        {
          control_state = USB_RECEIVE;  // set command with OUT token
        }
      } // set command with data
    } // else set command
  } // if setup packet

  else
  {
    if (control_state == USB_RECEIVE)
    {
      wtemp = isp_GetCtrlEndpoint(ControlData.dataBuffer + ControlData.wCount, EP_CTRL_PACKET_SIZE);

      ControlData.wCount += wtemp;
      if( (wtemp != EP_CTRL_PACKET_SIZE) || (ControlData.wCount >= ControlData.wLength))
      {
        control_state = USB_IDLE;
        control_handler();
      }
    }
    else
    {
      control_state = USB_IDLE;
    }
  }
}   // interrupt_CTRL_OUT



void  interrupt_CTRL_IN(void)
{
UINT16  tosend, sent;
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_CTRL_IN);    // clears interrupt bit

  printd(" CTRL_IN: ctrl=%02X, ep_status=%02Xh - ", control_state, ep_status);

  if (control_state != USB_TRANSMIT)
  {
    return;
  }

  tosend = ControlData.wLength - ControlData.wCount;

  sent = isp_WriteCtrlEndpoint((ControlData.pData + ControlData.wCount), tosend);
  ControlData.wCount += sent;

  if ( (tosend < EP_CTRL_PACKET_SIZE) && (tosend == sent) )
    control_state = USB_IDLE;

}   // interrupt_CTRL_IN



void  interrupt_MAIN_IN(void)
{
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_MAIN_IN);    // clears interrupt bit

  printd1("MAIN_IN: ep_status=%02Xh. ", ep_status);

  FeedBulkINFifo(&MAINBuffer);   // feed buffer; because of spinlock in FeedBulkINBuffer always get a new status there!

}   // interrupt_MAIN_IN



void interrupt_MAIN_OUT(void)
{
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_MAIN_OUT);   // clears interrupt bit

  printd("MAIN_OUT: ep_status=%02Xh\n", ep_status);

  GetBulkOUTFifo(&MAINBuffer, ep_status);

} //  interrupt_MAIN_OUT



void interrupt_ISO1(void)
{
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_ISO1);       // clears interrupt bit

  FeedIsoFifo(0);   // feed fifo buffer

}   // interrupt_ISO1


void interrupt_ISO2(void)
{
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_ISO2);       // clears interrupt bit

  FeedIsoFifo(1);   // feed fifo buffer

}   // interrupt_ISO2


void interrupt_ISO3(void)
{
UINT8   ep_status;

  ep_status = usb_wrc_rd1b(ISP_RD_EPSR_ISO3);       // clears interrupt bit

  FeedIsoFifo(2);   // feed fifo buffer

}   // interrupt_ISO3


void isp_restart(void)
{
  isp_init(TRUE);
}



UINT32 delayc(UINT32 value)
{
UINT32  i, j;

  j = 0;

  if (value)
    for (i = 0; i < value; i++)
      j += j;

  return j;

} // delayc

