/***************************************************************************
                                  usbdev.h
                              -------------------
    project:  USB device for Philips ISP1181A

    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __USBDEV__
#define __USBDEV__

#include  <linux/module.h>
#include  <linux/kernel.h>
#include  <linux/init.h>

// We use this driver in a non open system. So we can "allow" ourself to use
//   static Major etc numbers. Please be aware that you keep an eye on this,
//   if you use this driver on a public system.

#define MODULE_NAME         "usbdev"
#define MODULE_INFO_AUTHOR  "Siegfried Wessler, T-V, HBM GmbH"
#define MODULE_INFO_DESC    "Driver for HBM MGCplus: CP42 access as USB device via ISP1181A"
#define MODULE_INFO_SUPPORT "HBM CP42 with USB chip Philips ISP1181A"
#define MODULE_INFO_LICENSE "GPL"

#define DEV_MAJOR     196   // unused in "/lib/modules/2.4.4-4GB/build/Documentation/devices.txt"
                            // used in Kernel 2.4.18 by Tormenta T1
                            // @@@ check before publishing!

// In my "file:/lib/modules/2.4.4-4GB/build/Documentation/ioctl-number.txt"
// magic no. 0x9F is not mentioned. I try using this static.
// Be aware that Linux on CP22/CP42 will be in a close environment, so strange magics should
// usually not make any problem or at least can be handled by the developers of CP22/CP42,
// who should always know, who's modules are running in this embedded system.
#define USB_IOC_MAGIC         0x9F

#define USB_IOC_SET_SERNUM    _IOW(USB_IOC_MAGIC, 1, 16)
#define USB_IOC_SET_BUFSIZE   _IOW(USB_IOC_MAGIC, 2, 16)
#define USB_IOC_SET_FLAGS     _IOW(USB_IOC_MAGIC, 3, 16)
#define USB_IOC_MAX_NR          3

#define USB_VENDOR_ID         0x10D1    // HBM unique vendor id, never change!
#define USB_SPEC_ID           0x0110    // USB 1.10 compliant

#define USB_PRODUCT_ID        0x0301    // CP42 official, new product id to ensure loading of newer windows host driver
#define USB_REVISION_ID       0x0101    // CP42 with ISP1181A


#define IOCTL_BUFSIZE_MASK    0x0FFF    // Bits 11..0
#define IOCTL_SOF_BIT         0x1000    // Bit 12

#define IOCTL_NONBLOCKING_BIT 0x0001    // Bit 0

// **************************************

#define MAX_DEVICES           4   // number of minor device nodes for "/dev/usbdev"



extern int usb_ioctl(struct inode *inodep, struct file *filep, unsigned int cmd, unsigned long arg);
extern int usb_open(struct inode *inodp, struct file *filp);
extern int usb_close(struct inode *inodp, struct file *filp);
extern unsigned int usb_poll(struct file *filp, poll_table *wait);


extern ssize_t usb_read(struct file *filp, char *userBuffer, size_t userCount, loff_t *loff);
extern ssize_t usb_write(struct file *filp, const char *userBuffer, size_t userCount, loff_t *loff);

#endif
