/*
 *     ##################################################################
 *     #                                                                #
 *     #  Copyright(c) 1999 - 2001,  Analogic Corporation.  All rights  #
 *     #  reserved worldwide.                                           #
 *     #                                                                #
 *     #  This  document contains information proprietary  to Analogic  #
 *     #  Corporation.  If this product is acquired by or on behalf of  #
 *     #  a unit or agency of the United States Government the follow-  #
 *     #  ing applies: (a) This product was not developed with govern-  #
 *     #  ment funds;  (b) is a  trade secret of  Analogic Corporation  #
 *     #  for all purposes of the Freedom of Information Act;  and (c)  #
 *     #  is "Commercial Computer Software".  For units of the Depart-  #
 *     #  ment of Defense (DoD),  this software is  provided only with  #
 *     #  "Restricted Rights"  as defined in the DoD supplement to the  #
 *     #  Federal Acquisition Regulations, 52.227-7013(c)(I)(ii). Use,  #
 *     #  duplication,  or disclosure  is  subject to the restrictions  #
 *     #  set forth in subdivision (c)(ii)  of the Rights in Technical  #
 *     #  Data and Computer Software clause at 52-227-7013.             #
 *     #                                                                #
 *     #  Use, duplication, or disclosure of this proprietary document  #
 *     #  without  the express permission of  Analogic  Corporation is  #
 *     #  prohibited.                                                   #
 *     #                                                                #
 *     #                   C O N F I D E N T I A L                      #
 *     #  The information contained  in or upon  this document  is the  #
 *     #  property  of  Analogic Corporation and  is considered to  be  #
 *     #  proprietary  and  may  not  be used by any recipient without  #
 *     #  the specific written permission of Analogic Corporation.      #
 *     #                                                                #
 *     ##################################################################
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <termios.h>
#include "gpib.h"

#define ERRORS(s) fprintf(stderr,"Error from line %d, file %s, call %s(), (%s)\n",__LINE__,__FILE__,s,strerror(errno))

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
static char gpib[]="/dev/gpib";

struct{const char *cmd; int id; int pars; const char *text;} cmdtab[] =
{
  {"quit", CMD_QUIT, 0, "quit this program"},
  {"help", CMD_HELP, 0, "get this command help"},
  {"sadr", CMD_SADR, 1, "set the gpib adress"},
  {"gadr", CMD_GADR, 0, "get the gpib adress"},
  {"sreg", CMD_SREG, 2, "set a gpib register"},
  {"greg", CMD_GREG, 1, "get a gpib register"},
  {"rdwr", CMD_RDWR, 0, "read data from the gpib and write them back"},
  {NULL, CMD_NCMD, 0, NULL}
};

int cmd_help(void)
{
  int i=-1;

  fprintf(stdout, "CMD  PARS  TEXT\n");

  while(cmdtab[++i].cmd)
  {
    fprintf(stdout, "%s  %d    %s\n", cmdtab[i].cmd, cmdtab[i].pars, cmdtab[i].text);
  }

  return 0;
}

int get_cmd(char *cmd, int *id, int *pars)
{
  int i=-1;

  while(cmdtab[++i].cmd)
    if(!strcmp(cmd, cmdtab[i].cmd))
       break;

  *id = cmdtab[i].id;
  *pars = cmdtab[i].pars;

  return(cmdtab[i].id);
}

int main(int args, char *argv[])
{
    char    buf[L_BUF];
//    fd_set  rfds;
    int     fd, flags, len, param, ret_val, written;
    int     run = 1;
    char    cmd[10];
    unsigned int par1, par2;
    int     cmd_id, cmd_pars;

    if((fd = open(gpib, O_RDWR | O_NDELAY)) < 0)
    {
      ERRORS("open");
    }

    if((flags = fcntl(fd, F_GETFL)) < 0)
        ERRORS("fcntl");
    flags &= ~O_NDELAY;               // set to blocking mode
    if(fcntl(fd, F_SETFL, flags) < 0)
        ERRORS("fcntl");


    while(run)
    {
      fprintf(stdout, "waiting for command <scmd (ipar1) (ipar2) CR>\n");
      buf[0] = 0;
      fgets(buf, sizeof(buf), stdin);
      if((ret_val=sscanf(buf, "%s %02x %02x", cmd, &par1, &par2)) == EOF)
      {
        fprintf(stdout, "error: try again!\n");
        continue;
      }

      if(get_cmd(cmd, &cmd_id, &cmd_pars) == CMD_NCMD)
        continue;

      if(ret_val != cmd_pars+1)
      {
        fprintf(stdout, "wrong parameter count: %s needs %d!\n", cmd, cmd_pars);
        continue;
      }

      switch(cmd_id)
      {
        case CMD_HELP:
          cmd_help();
          break;

        case CMD_GADR:
          if(ioctl(fd, GPIB_GET_PGPIB_ADDR, &par1) < 0)
            ERRORS("ioctl");
          else
            fprintf(stdout, "%s: 0x%02x\n", cmd, par1);
          break;

        case CMD_SADR:
          if(ioctl(fd, GPIB_SET_PGPIB_ADDR, &par1) < 0)
            ERRORS("ioctl");
          else
            fprintf(stdout, "%s: 0x%02x done\n", cmd, par1);
          break;

        case CMD_SREG:
          param = par1;
          param |= par2<<16;
          if(ioctl(fd, GPIB_SET_REGISTER, &param) < 0)
            ERRORS("ioctl");
          else
            fprintf(stdout, "%s: 0x%02x 0x%02x done\n", cmd, par1, par2);
          break;

        case CMD_GREG:
          if(ioctl(fd, GPIB_GET_REGISTER, &par1) < 0)
            ERRORS("ioctl");
          else
            fprintf(stdout, "%s: 0x%02x\n", cmd, par1);
          break;

        case CMD_RDWR:
          if((len=read(fd, buf, L_BUF)) >= 0)
          {
            written = 0;

            par1 = 1;
            ioctl(fd, GPIB_SET_EOIFLG, &par1);

            while(len)
            {
              if((ret_val=write(fd, &buf[written], len)) >= 0)
              {
                written += ret_val;
                len -= ret_val;
              }
              else
              {
                ERRORS("write");
                break;
              }
            }

//            par1 = 0;
//            ioctl(fd, GPIB_SET_EOIFLG, &par1);

            fprintf(stdout, "%s: received %d and sended %d data bytes\n", cmd, len+written, written);
          }
          else
            ERRORS("read");
          break;

        case CMD_QUIT:
          run = 0;
          fprintf(stdout, "bye bye...\n");

        default:
          break;
      }

    }
//    param = 0;
//    if(ioctl(fd, GPIB_ADDR_WRITE, &param) < 0)    /* Set our address to zero */
//        ERRORS("ioctl");

//    if(ioctl(fd, GPIB_INIT, NULL)<0)              /* Initialize controller   */
//        ERRORS("ioctl");

//    if(ioctl(fd, GPIB_GET_GPIB_ADDR, &param) < 0) /* Get our address         */
//        ERRORS("ioctl");

//    if(ioctl(fd, GPIB_INPUT_FLUSH, NULL) < 0)
//        ERRORS("ioctl");

//    if(ioctl(fd, GPIB_OUTPUT_FLUSH, NULL) < 0)
//        ERRORS("ioctl");

//    if(param != 0)
//    {
//        fprintf(stderr, "Address register not correct (0x%02x)\n", param);
//        exit(EXIT_FAILURE);
//    }

//    if(ioctl(fd, GPIB_INP_STS, &param) < 0)
//        ERRORS("ioctl");
//    if(param != 0)
//    {
//        fprintf(stderr, "Input status not correct (%d)\n", param);
//        exit(EXIT_FAILURE);
//    }

//    if(ioctl(fd, GPIB_OUT_STS, &param) < 0)
//        ERRORS("ioctl");
//    if(param != 0)
//    {
//        fprintf(stderr, "Output status not correct (%d)\n", param);
//        exit(EXIT_FAILURE);
//    }

//    if(ioctl(fd, GPIB_SET_DUMB_MODE, NULL)< 0)
//        ERRORS("ioctl");

//    for(;;)
//    {
//        len = sizeof(buf);
//        fprintf(stderr,"Calling write...");
//        if((ret_val = write(fd, buf, len)) != len)
//            fprintf(stderr, "returned %d\n", ret_val);
//        else
//            fprintf(stderr, "okay\n");
//        for(;;)
//        {
//            FD_ZERO(&rfds);
//            FD_SET(fd, &rfds);
//            fprintf(stderr, "Calling select()\n");
//            select(fd +1, &rfds, NULL, NULL, NULL);
//            fprintf(stderr, "Returned from select()\n");
//            if(FD_ISSET(fd, &rfds))
//            {
//                fprintf(stderr, "Calling read()\n");
//                len = read(fd, buf, 0x1000);
//                fprintf(stderr, "%d   \n", len);
//                break;
//            }
//        }
//    }

    close(fd);
    exit(EXIT_SUCCESS);
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
