/***************************************************************************
                          isr.c  -  description
                             -------------------
    begin                : Mon Sep 15 2003
    copyright            : (C) 2003 by Arno Geissel
    email                : geissel@t-c-linux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <asm/spinlock.h>
#include <asm/uaccess.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>

 #if CONFIG_MODVERSIONS==1
 #define MODVERSIONS
 #include <linux/modversions.h>
 #endif

#include <linux/pci.h>
#include <linux/interrupt.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/cisreg.h>
#include <pcmcia/ds.h>

#include "global.h"
#include "tnt4882_regs.h"
#include "buffer.h"
#include "vfs_io.h"
#include "isr.h"

DECLARE_TASKLET (tasklet_after_interrupt, after_gpib_interrupt, 0);

unsigned char isr[4];
unsigned char adsr;
int rx_end;
int adressed, common_adressed;

// This is the interrupt service routine
void gpib_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
  #ifdef PRINTK_DEBUG
  int loop = 0;
  #endif

  #ifdef SHOW_BITS
   outb(inb(0x378)|0x02, 0x378);
  #endif

  DISABLE_INTERRUPT;

  while((isr[3] = board_in(Board.regs->ISR3)) & ISR3_INT)
  {
    #ifdef PRINTK_DEBUG
    if(loop++ > 10)
      printk("sticky interrupt ISR3: %02x\n", isr[3]);
    #endif

    while((isr[2] = board_in(Board.regs->ISR2)) & ISR2_INT)
    {
      #ifdef PRINTK_DEBUG
      if(loop++ > 10)
        printk("sticky interrupt ISR2: %02x\n", isr[2]);
      #endif

      isr[1] = board_in(Board.regs->ISR1);
      isr[0] = board_in(Board.regs->ISR0);

      if(isr[2] & ISR2_ADSC)
      {
        adsr = board_in(Board.regs->ADSR);

        if(!(isr[3] & ISR3_STOP))
          board_out(Board.regs->CMDR, STOP);   // stop previous gpib transfer as early as possible

        if (adsr & ADSR_LA)            // TNT4882 is becoming Listener
        {
          common_adressed = LISTENER;
        }
        else if (adsr & ADSR_TA)       // TNT4882 is becoming Talker
        {
          common_adressed = TALKER;
        }
        else                           // TNT4882 is becoming Unadressed
        {
          common_adressed = UNADRESSED;
        }

        rx_end = FALSE;                // init rx_end
      }

      if(isr[1] & ISR1_END_RX)         // we received END condition caused by EOI
      {
        rx_end = TRUE;
      }

      if(isr[0] & ISR0_TO)
      {
        // load timer
        board_out(Board.regs->AUXMR, Board.mirr->auxrj = TIMEOUT);
      }
    }

    if (isr[3] & (ISR3_NEF | ISR3_NFF))
    {
      board_out(Board.regs->IMR3, Board.mirr->imr3 &= ~IMR3_NEF_IE);

      board_out(Board.regs->IMR3, Board.mirr->imr3 &= ~IMR3_NFF_IE);
    }
  }

  tasklet_schedule(&tasklet_after_interrupt);

  //after_gpib_interrupt(0);
  //ENABLE_INTERRUPT;

  #ifdef SHOW_BITS
   outb(inb(0x378)&~0x02, 0x378);
  #endif
}

void set_gpib_cnt(int count)
{
  unsigned char *byte;

  byte = (unsigned char *) &count;
  count = -count;

  board_out( Board.regs->CNT0,( unsigned char ) *byte++ );
  board_out( Board.regs->CNT1,( unsigned char ) *byte++ );
  board_out( Board.regs->CNT2,( unsigned char ) *byte++ );
	board_out( Board.regs->CNT3,( unsigned char ) *byte );
}

void after_gpib_interrupt (unsigned long arg)         //this is the tasklet
{
  unsigned char byte[33], count;
  int rego_flg;
  int ret;
  unsigned char STS2;

  #ifdef SHOW_BITS
   outb(inb(0x378)|0x04, 0x378);
  #endif

  rego:
  rego_flg = 0;

  if(adressed == TALKER && common_adressed == TALKER)
  {
    if (main_buffer.send_count)
    {
      ret = write_on_gpib();

      //if FIFO is empty and one or two bytes are left in out_buffer assert EOI line before sending last bytes
      if(((main_buffer.send_count == 1) || (main_buffer.send_count == 2)) &&
           !(board_in(Board.regs->STS2) & (STS2_AEFN | STS2_BEFN)))
      {
        if(main_gpib.write_done && main_gpib.eoi)
        {
          #ifdef PRINTK_DEBUG
          printk("EOI-SET: %d\n", main_buffer.send_count);
          #endif

          set_gpib_cnt(main_buffer.send_count);                        // set the transfer counter to 1
        }

        count = 0;
        while (main_buffer.send_count)
        {
          if (main_buffer.psend_outbuf >= &main_buffer.out_buffer[OUTBUF_SIZE])
            main_buffer.psend_outbuf = main_buffer.out_buffer;         // repeat buffer

          main_buffer.send_count--;
          byte[count++] = *(main_buffer.psend_outbuf++);
        }

        if(count == 1)
          board_out(Board.regs->FIFOB, *byte);
        else
          board_outw(Board.regs->FIFOB, *(unsigned short*)byte);
      }
      if(ret)                                                          // only if something is going on
        board_out(Board.regs->IMR3, Board.mirr->imr3 |= IMR3_NFF_IE);
    }
  }
  else if(adressed == LISTENER)
  {
    ret = read_from_gpib();

    if(rx_end)
    {
      board_out(Board.regs->CMDR, STOP);                              // stop previous gpib transfer on END received

      if(*(main_buffer.precv_inbuf-1) != C_NL)                        // we have to generate a NL ourself bec. interpreter
      {
        board_out(Board.regs->CFG,Board.mirr->cfg &= ~CFG_IN);        // set direction mode "write on gpib"
        board_out(Board.regs->FIFOB, C_NL);                           // put NL into the fifo, will be read next turn
        board_out(Board.regs->CFG, Board.mirr->cfg |= CFG_IN);        // set direction mode "read from gpib"
        #ifdef PRINTK_DEBUG
        printk("generated NL\n");
        #endif
      }

      rx_end = FALSE;
    }

    if(ret)
      board_out(Board.regs->IMR3, Board.mirr->imr3 |= IMR3_NEF_IE);
  }

  if(adressed != common_adressed)
  {
    if(common_adressed == LISTENER)  // we become listener
    {
      //Rescue unsent bytes out of Fifo into outbuffer before writing new received data into Fifo
      if(board_in(Board.regs->STS2) & (STS2_BEFN | STS2_AEFN))
      {
        count=0;
        while ((STS2=board_in(Board.regs->STS2)) & (STS2_BEFN | STS2_AEFN))
        {
          if(STS2 & STS2_AEFN)
          {
            *(unsigned short*)(byte+count) = board_inw(Board.regs->FIFOB);
            count+=2;
          }
          else
          {
            byte[count] = board_in(Board.regs->FIFOB);
            count++;
          }
        }
        #ifdef PRINTK_DEBUG
        byte[count] = 0;
        printk("rescued untransfered data: %s\n", byte);
        #endif
        while(count)
        {
          count--;
          *(--main_buffer.psend_outbuf) =  byte[count];

          if (main_buffer.psend_outbuf == main_buffer.out_buffer)
            main_buffer.psend_outbuf = &main_buffer.out_buffer[OUTBUF_SIZE];
        }
      }

      set_gpib_cnt(GPIB_MAX_CNT);                               // set the transfer counter
      Board.mirr->cfg &= ~CFG_16_8N;                            // set to 8 Bit width
      board_out(Board.regs->CFG, Board.mirr->cfg |= CFG_IN);    // set direction mode "read from gpib"
      //board_out(Board.regs->CMDR, GO);                        // start transfer
    }
    else if(common_adressed == TALKER)  // we become talker
    {
      if(adressed == LISTENER)
        read_from_gpib();

      set_gpib_cnt(GPIB_MAX_CNT);                               // set the transfer counter
      Board.mirr->cfg |= CFG_16_8N;                             // set to 16 Bit width
      board_out(Board.regs->CFG,Board.mirr->cfg &= ~CFG_IN);    // set direction mode "write on gpib"
      board_out(Board.regs->CMDR, GO);                          // start transfer
    }

    rego_flg = TRUE;
    adressed = common_adressed;
  }

  if(/*adressed == TALKER ||*/ adressed == LISTENER)          // we all times want to recv if we are listener
    board_out(Board.regs->CMDR, GO);                          // start transfer

  if(rego_flg)
    goto rego;

  ENABLE_INTERRUPT;

  #ifdef SHOW_BITS
   outb(inb(0x378)&~0x04, 0x378);
  #endif
}



int read_from_gpib(void)
{
  int read = 0;

  #ifdef PRINTK_DEBUG
  char showbuf[32];
  #endif

  spin_lock(&lock_gpib_in);

  while(board_in(Board.regs->STS2) & STS2_BEFN)       // read while Fifo not empty
  {
    if(main_buffer.precv_inbuf >= &main_buffer.in_buffer[INBUF_SIZE]) // ring-buffer, if max. Adress has been reached
    {                                                                 // set receive pointer to start-adress
      if (main_buffer.pread_inbuf > (main_buffer.in_buffer+2))
        main_buffer.precv_inbuf = main_buffer.in_buffer;              // repeat buffer
      else
      {
        printk("Buffer full recv\n");
        break;
      }
    }

    if(main_buffer.precv_inbuf >= main_buffer.pread_inbuf)    // fill buffer, if array size is not exceeded
    {
      main_buffer.recv_count++;

      #ifdef PRINTK_DEBUG
      *(main_buffer.precv_inbuf++) = showbuf[read++] = board_in(Board.regs->FIFOB);   // fill in_buffer with Fifo-Bytes
      #else
      *(main_buffer.precv_inbuf++) = board_in(Board.regs->FIFOB);   // fill in_buffer with Fifo-Bytes
      read++;
      #endif
    }
    else if(main_buffer.precv_inbuf < (main_buffer.pread_inbuf-2)) // security distance
    {
      main_buffer.recv_count++;

      #ifdef PRINTK_DEBUG
      *(main_buffer.precv_inbuf++) = showbuf[read++] = board_in(Board.regs->FIFOB);   // fill in_buffer with Fifo-Bytes
      #else
      *(main_buffer.precv_inbuf++) = board_in(Board.regs->FIFOB);
      read++;
      #endif
    }
    else
    {
      printk("Buffer full recv\n");
      break;
    }
  }           // end while

  #ifdef PRINTK_DEBUG
   if(read)
   {
     showbuf[read] = 0;
     printk("read_from_gpib %d %d %s\n", main_buffer.recv_count, read, showbuf);
   }
  #endif

  if(main_buffer.recv_count)
    wake_up_interruptible(&main_buffer.waitqueue);

  spin_unlock(&lock_gpib_in);

  return read;
}



int write_on_gpib(void)
{
  unsigned char  STS2;
  int            i;
  unsigned char  word[2];
  int            written = 0;

  spin_lock(&lock_gpib_out);

  #ifdef SHOW_BITS
   outb(inb(0x378)|0x08, 0x378);
  #endif

  while ( ((STS2=board_in(Board.regs->STS2)) & STS2_BFFN) &&                // hihi!
           (STS2 & STS2_AFFN))                                              // while Fifos are not full
  {
    if(main_buffer.send_count <= 2)                                         // we want to leave one byte in the buffer
      goto leave;

    for (i=0; i<2; i++)
    {
      if (main_buffer.psend_outbuf >= &main_buffer.out_buffer[OUTBUF_SIZE])
        main_buffer.psend_outbuf = main_buffer.out_buffer;                  // repeat buffer

      written++;
      main_buffer.send_count--;
      word[i] = *(main_buffer.psend_outbuf++);
    }

    board_outw(Board.regs->FIFOB, *(unsigned short*)word);
  }

  leave:

  #ifdef SHOW_BITS
   outb(inb(0x378)&~0x08, 0x378);
  #endif

  spin_unlock(&lock_gpib_out);

  #ifdef PRINTK_DEBUG
  if(written)
    printk("write_on_gpib %d\n", written);
  #endif

  return written;
}
