/***************************************************************************
                          vfs_io.c  -  description
                             -------------------
    begin                : Wed Sep 10 2003
    copyright            : (C) 2003 by Arno Geissel
    email                : geissel@t-c-linux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <asm/spinlock.h>
#include <asm/io.h>

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>

 #if CONFIG_MODVERSIONS==1
 #define MODVERSIONS
 #include <linux/modversions.h>
 #endif

#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/cistpl.h>


#include <pcmcia/cisreg.h>
#include <pcmcia/ds.h>

#include "global.h"
#include "vfs_io.h"
#include "tnt4882_regs.h"
#include "buffer.h"
#include "gpib.h"
#include "isr.h"

tDevice main_gpib;

spinlock_t  lock_gpib_out;
spinlock_t  lock_gpib_in;

/* gpib_open: called by the usermode open() systemcall */
int gpib_open(struct inode *inodp, struct file *filp)
{
  unsigned short minor_no;
  tDevice        *dev;
  int            use_cnt;

  minor_no = MINOR(inodp->i_rdev);
  dev = &main_gpib;
  if(dev == NULL)
    return -ENODEV;

  use_cnt = MOD_IN_USE;

  //if(use_cnt <= 1)             // to be able to open it additional in tester
  //{
    DEBUG("gpib_open %d\n", use_cnt);
    board_out(Board.regs->CMDR, RESET_FIFO);  //Reset the FIFOs
    init_fifo_buffer(&main_buffer);

    dev->flags |= (FLAG_INTERFACE_OPEN);
    dev->minor_no = minor_no;
    dev->eoi = 0;
    dev->write_done = FALSE;
    sema_init(&dev->sem, 1);



  //}
  //else
  //{
  //  DEBUG("gpib_open %d\n", use_cnt);
  //}

  filp->private_data = dev;

  MOD_INC_USE_COUNT;
  return 0;
}


/* gpib_release: called by the usermode close() systemcall */
int gpib_release(struct inode *inodp, struct file *filp)
{
  tDevice *dev;
  int     use_cnt;

  dev = filp->private_data;
  if (dev == NULL)
    return -ENODEV;

  //down (&dev->sem);   // lock the device
  MOD_DEC_USE_COUNT;
  use_cnt = MOD_IN_USE;

  //if(use_cnt <= 1)
  //{
    DEBUG("gpib_release %d\n", use_cnt);

    dev->flags &= ~(FLAG_INTERFACE_OPEN);
    if (dev->minor_no == 0)
      wake_up_interruptible(&main_buffer.waitqueue);  // return outstanding read calls
  //}
  //else
  //{
  //  DEBUG("gpib_release %d\n", use_cnt);
  //}

  //up (&dev->sem);   // unlock the device


  return 0;
}


/* gpib_write: called by the usermode write() systemcall */
ssize_t gpib_write( struct file *filp, const char *buffer, size_t length, loff_t *wtf )
{
  tDevice *dev;
  int free_space;
  int bytes_written = 0;
  int mlength;
  unsigned long flags;

  DEBUG("gpib_write %d\n", length);

  dev = filp->private_data;
  if (dev == NULL)
    return -ENODEV;

  //down(&dev->sem);

  spin_lock_irqsave(&lock_gpib_out, flags);

  #ifdef SHOW_BITS
   outb(inb(0x378)|0x40, 0x378);
  #endif

  mlength = length;

  //special case, if outbuffer is at its maximum boundary
  if(main_buffer.pwrite_outbuf >= &main_buffer.out_buffer[OUTBUF_SIZE])
  {
    if(main_buffer.psend_outbuf > (main_buffer.out_buffer+2))
    {
      main_buffer.pwrite_outbuf = main_buffer.out_buffer;
    }
    else
    {
      #ifdef PRINTK_DEBUG
      printk("Outbuffer full write 1\n");
      #endif

      goto done;          // nothing could be written
    }
  }

  if(main_buffer.psend_outbuf <= main_buffer.pwrite_outbuf)         // end of buffer is the nearest border
  {
    free_space = &main_buffer.out_buffer[OUTBUF_SIZE] - main_buffer.pwrite_outbuf;  // free memory to upper boundary

    if (free_space > mlength)               // enough free memory to upper boundary of buffer
    {
      copy_from_user(main_buffer.pwrite_outbuf, buffer, mlength);
      main_buffer.pwrite_outbuf += mlength;
      //atomic_add_in_place(main_buffer.pwrite_outbuf, mlength);
      bytes_written += mlength;
    }
    else //if (free_space_to_max <= mlength)  // not enough memory to upper boundary for requested length
    {
      copy_from_user(main_buffer.pwrite_outbuf, buffer, free_space); // write into the available space
      main_buffer.pwrite_outbuf += free_space;          // we are at the end
      //atomic_add_in_place(main_buffer.pwrite_outbuf, free_space);
      mlength -= free_space;                             // calculate the number of bytes that could not be written
      bytes_written += free_space;

      if ((main_buffer.psend_outbuf > (main_buffer.out_buffer+2)))
        main_buffer.pwrite_outbuf = main_buffer.out_buffer;
      else
      {
        #ifdef PRINTK_DEBUG
        printk("Outbuffer full write 2\n");
        #endif

        goto done;
      }

      // now we are behind psend
      behind_psend:
      free_space = main_buffer.psend_outbuf - main_buffer.pwrite_outbuf - 2; // calculate the free memory between pwrite and psend

      if(free_space > mlength)                                               // there is enough memory for the not written bytes
      {
        copy_from_user(main_buffer.pwrite_outbuf, buffer + bytes_written, mlength);
        main_buffer.pwrite_outbuf += mlength;
        //atomic_add_in_place(main_buffer.pwrite_outbuf, mlength);
        bytes_written += mlength;
      }
      else //if (free_space <= mlength)                   // there is not enough memory, only use the available space
      {
        copy_from_user(main_buffer.pwrite_outbuf, buffer + bytes_written, free_space);
        main_buffer.pwrite_outbuf += free_space;
        //atomic_add_in_place(main_buffer.pwrite_outbuf, free_space);
        bytes_written += free_space;
        #ifdef PRINTK_DEBUG
        printk("Outbuffer full write 3\n");
        #endif

        goto done;
      }

    }
  }
  else
  {
    goto behind_psend;
  }

  done:

  if(bytes_written == length)         // all is done
    dev->write_done = TRUE;

  main_buffer.send_count += bytes_written;

  #ifdef SHOW_BITS
   outb(inb(0x378)&~0x40, 0x378);
  #endif

  spin_unlock_irqrestore(&lock_gpib_out, flags);

  //up(&dev->sem);

  return bytes_written;
}


/* gpib_read: called by the usermode read() systemcall */
ssize_t gpib_read( struct file *filp, char *buffer, size_t length, loff_t *wtf )
{
  tDevice *dev;
  int avail_bytes, copied_bytes=0;
  int count;
  unsigned long flags;

  dev = filp->private_data;
  if (dev == NULL)
    return -ENODEV;

  //down(&dev->sem);

  if (length < main_buffer.recv_count)
    avail_bytes = length;
  else
    avail_bytes = main_buffer.recv_count;

  if (filp->f_flags & O_NONBLOCK)
  {
    if(!avail_bytes)
      goto done;
  }
  else
  {
    if(!avail_bytes)
    {
      DEBUG("gpib_read BLOCK\n");
      wait_event_interruptible(main_buffer.waitqueue, main_buffer.pread_inbuf != main_buffer.precv_inbuf);

      //if(!(dev->flags & FLAG_INTERFACE_OPEN))
      //  return -ENODEV;
    }
  }

  spin_lock_irqsave(&lock_gpib_in, flags);

  if (length < main_buffer.recv_count)
    avail_bytes = length;
  else
    avail_bytes = main_buffer.recv_count;

  // if in_buffer reached its max. element; we use a ring buffer and we jump to the start element of the buffer
  if ((main_buffer.pread_inbuf + avail_bytes) > &main_buffer.in_buffer[INBUF_SIZE])
  {
    count = &main_buffer.in_buffer[INBUF_SIZE] - main_buffer.pread_inbuf;
    if (count)
      copy_to_user(buffer, main_buffer.pread_inbuf, count);
    copied_bytes += count;
    main_buffer.pread_inbuf += count;
    //atomic_add_in_place(main_buffer.pread_inbuf, count);

    count = avail_bytes - copied_bytes;
    if (count)
    {
      main_buffer.pread_inbuf = main_buffer.in_buffer;       // buffer repeat
      copy_to_user(buffer+copied_bytes, main_buffer.pread_inbuf, count);
      copied_bytes += count;
      main_buffer.pread_inbuf += count;
      //atomic_add_in_place(main_buffer.pread_inbuf, count);
    }
  }
  else
  {   //in this case, in_buffer has not reached its max. element
    count = avail_bytes;
    copy_to_user(buffer, main_buffer.pread_inbuf, count);
    copied_bytes += count;
    main_buffer.pread_inbuf += count;
    //atomic_add_in_place(main_buffer.pread_inbuf, count);
  }

  main_buffer.recv_count -= copied_bytes;
  //atomic_sub_in_place(&main_buffer.recv_count, copied_bytes);

  if(!(filp->f_flags & O_NONBLOCK))
  {
    DEBUG("copied to user %d\n", copied_bytes);
  }

  spin_unlock_irqrestore(&lock_gpib_in, flags);

  done:

  //up(&dev->sem);

  return copied_bytes;
}




/* gpib_ioctl: called by the usermode ioctl() systemcall */
int gpib_ioctl( struct inode* inode, struct file* filp, unsigned int cmd, unsigned long arg )
{
  tDevice *dev;
  int tmp, tmp2;

  dev = filp->private_data;
  if (dev == NULL)
    return -ENODEV;

  //down(&dev->sem);

  switch(cmd)
  {
    case GPIB_SET_EOIFLG:
      if(!!get_user(tmp, (int *)arg))
        return -EFAULT;
      DEBUG("GPIB_SET_EOIFLG: %d\n", (unsigned char)tmp);
      dev->eoi = (unsigned char)tmp;
      dev->write_done = FALSE;
      break;

    case GPIB_GET_EOIFLG:
      tmp = (int)dev->eoi;
      DEBUG("GPIB_GET_EOIFLG: %d\n", (unsigned char) tmp);
      if(!!put_user(tmp, (int *)arg))
        return -EFAULT;
      break;

    case GPIB_SET_PGPIB_ADDR:
      if(!!get_user(tmp, (int *)arg))
        return -EFAULT;
      DEBUG("GPIB_SET_GPIB_ADDR: %d\n", (unsigned char)(tmp & 0x1F));
      board_out( Board.regs->ADR, (Board.pad = (unsigned char)(tmp & 0x1F)) );
      break;

    case GPIB_GET_PGPIB_ADDR:
      tmp = (int)board_in( Board.regs->ADR0 );
      DEBUG("GPIB_GET_PGPIB_ADDR: %d\n", (unsigned char) tmp);
      if(!!put_user(tmp, (int *)arg))
        return -EFAULT;
      break;

    case GPIB_SET_REGISTER:
      if(!!get_user(tmp, (int *)arg))
        return -EFAULT;
      DEBUG("GPIB_SET_REGISTER: 0x%02x, 0x%02x\n", (unsigned char)tmp, (unsigned char)(tmp>>16));
      board_out(Board.base+(unsigned char)tmp, (unsigned char)(tmp>>16));
      break;

    case GPIB_GET_REGISTER:
      if(!!get_user(tmp, (int *)arg))
        return -EFAULT;
      tmp2 = (int)board_in(Board.base+(unsigned char)tmp);
      DEBUG("GPIB_GET_REGISTER: 0x%02x, 0x%02x\n", (unsigned char)tmp, (unsigned char)(tmp2));
      if(!!put_user(tmp2, (int *)arg))
        return -EFAULT;
      break;
  }

  //up(&dev->sem);
  return 0;
}

/* gpib_poll: called by the usermode select() systemcall */
unsigned int gpib_poll(struct file *filp, poll_table *wait)
{
  tDevice *dev;
  unsigned int  mask;

  DEBUG("gpib_poll\n");

  dev = filp->private_data;
  if (dev == NULL)
    return -ENODEV;

  //down (&dev->sem);   // lock this object
  mask = 0;

  poll_wait(filp, &main_buffer.waitqueue, wait);
  if (main_buffer.pread_inbuf != main_buffer.precv_inbuf)
    mask |= (POLLIN | POLLRDNORM);       // device is readable
  if ((INBUF_SIZE - main_buffer.send_count) >= 2)
    mask |= (POLLOUT | POLLWRNORM);      // device is writeable

  //up (&dev->sem);   // unlock the device
  return mask;

  return 0;
}

/* gpib_fops: the file operation structure */
struct file_operations gpib_fops =
{
  owner:    THIS_MODULE,
  read:     gpib_read,
  write:    gpib_write,
  ioctl:    gpib_ioctl,
  open:     gpib_open,
  release:  gpib_release,
  poll:     gpib_poll,
};


/* initialize this device driver, allocate recources,
   but don't access the hardware at this point */
int init_gpib_driver(void)
{
  DEBUG("init_gpib_driver\n");

  main_gpib.flags &= ~(FLAG_INTERFACE_OPEN);

  // init semaphores (not really used in a gpib device driver)
  sema_init(&main_gpib.sem, 1);

  // init spinlocks
  spin_lock_init(&lock_gpib_in);
  spin_lock_init(&lock_gpib_out);

  /* Allocate space for private device-specific data */
  Board.regs = kmalloc(sizeof(sio_regs), GFP_KERNEL);
  if (!Board.regs)
    goto eexit;
  memset(Board.regs, 0, sizeof(sio_regs));

  Board.mirr = kmalloc(sizeof(sio_regsmirr), GFP_KERNEL);
  if (!Board.mirr)
    goto eexit;
  memset(Board.mirr, 0, sizeof(sio_regsmirr));

  init_fifo_buffer(&main_buffer);

  Board.pad = 4;      // HBM default GPIB Adress
  Board.sad = 0;      // no secondary adress

  SET_MODULE_OWNER(&gpib_fops);

  if ( (register_chrdev(DEV_MAJOR, MODULE_NAME, &gpib_fops)) != 0 )
  {
    printk("%s: register_chrdev failed.\n", MODULE_NAME);
    return -1;
  }

  return 0;

  eexit:
  if(Board.regs)
    kfree(Board.regs);
  if(Board.mirr)
    kfree(Board.mirr);

  return -1;
}

/* release the driver ressources */
void release_gpib_driver(void)
{
  DEBUG("release_gpib_driver\n");

  if (unregister_chrdev(DEV_MAJOR, MODULE_NAME) != 0)
    printk("%s: Unload of driver failed.\n", MODULE_NAME);

  if(Board.regs)
    kfree(Board.regs);
  if(Board.mirr)
    kfree(Board.mirr);
}
