/***************************************************************************
                          cpxxnmi.h  -  description
                             -------------------
    begin                : Mon Feb 5 2001
    copyright            : (C) 2001 by ag
    email                : arno.geissel@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 #ifndef __CPXXNMI_H__
 #define __CPXXNMI_H__

 #define NMI_MODULE
 #define MODULE_NAME  "cpxxnmi"

 // debug issues
 #define ALL_MSG "cpxxnmi: "
 #define ALL_DEBUG

 #undef PDEBUG             /* undef it, just in case */
 #ifdef ALL_DEBUG
 #  define __static
 #  ifdef __KERNEL__
      /* This one if debugging is on, and kernel space */
 #    define PDEBUG(fmt, args...) printk( /*KERN_DEBUG*/ ALL_MSG fmt, ## args)
 #  else
      /* This one for user space */
 #    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
 #  endif
 #else
 #  define PDEBUG(fmt, args...) /* not debugging: nothing */
 #  define __static static
 #endif

 #undef PDEBUGG
 #define PDEBUGG(fmt, args...) PDEBUG(fmt, ## args) /* nothing: it's a placeholder */

 // debugging issues
 #define BREAKPOINT   __asm__ __volatile__("int $1")

 // Showbits
 //#define SHOWBIT_DEBUG

 // types
 typedef long             INT32;
 typedef unsigned long    UINT32;
 typedef short            INT16;
 typedef unsigned short   UINT16;
 typedef char             INT8;
 typedef unsigned char    UINT8;
 typedef float            FLOAT32;
 typedef double           FLOAT64;

 // device issues
 #define DEV_MAJOR                  42
 #define CPXX_IOC_MAGIC             'k'
 #define CPXX_IOC_GFIFOSIZE         _IOR(CPXX_IOC_MAGIC, 1, 1000)
 #define CPXX_IOC_SGA12DATA         _IOW(CPXX_IOC_MAGIC, 2, 1000)
 #define CPXX_IOC_GGA12DATA         _IOR(CPXX_IOC_MAGIC, 3, 1000)
 #define CPXX_IOC_DISABLE_NMI       _IOR(CPXX_IOC_MAGIC, 4, 1000)
 #define CPXX_IOC_ENABLE_NMI        _IOR(CPXX_IOC_MAGIC, 5, 1000)
 #define CPXX_IOC_CPXX_BEEP         _IOR(CPXX_IOC_MAGIC, 6, 1000)

 #define BEEP_OK   1200
 #define BEEP_WARN 2400
 #define BEEP_ERR  4800

 #define TYPE(dev)             (MINOR(dev) >> 4)
 #define NUM(dev)              (MINOR(dev) & 0xf)
 #define CPXXNMI_NR_DEVS       1

 // FIFO issues
 #define FIFO_KBYTES            32768

 // PCI issues
 #define GA12_VENDOR            0x11E3
 #define GA12_DEVICE            0x10
 #define GA12_INDEX             0x00
 #define GA12_DATA_RAM_SIZE     0x100000

 // NMI issues
 #define DISABLE_NMI outb( inb(0x61) | 0x0c , 0x61 )      // SERRx always off
 #define ENABLE_NMI  outb( inb(0x61) & ~8, 0x61 )

 // data
 extern struct pci_dev *ga12_dev;
 extern u8     ga12_pci_irq;                // irq line of ga12
 extern void   *fifo_ram;                   // pointer to fifo_ram
 extern unsigned long fifo_size;            // size of allocated fifo memory
 extern wait_queue_head_t cpxxnmi_wait;     // waitqueue for blocking user read call operation

 // types
 typedef struct
 {
   unsigned   index;
   unsigned  size;
   void     *pdata;
 }S_TGA12DATA;

 // function prototypes
 void nmi_handler(void);
 int cpxxnmi_open(struct inode *inodp, struct file *filp);
 int cpxxnmi_release(struct inode *inodp, struct file *filp);
 int cpxxnmi_mmap(struct file *filp, struct vm_area_struct *vma);
 int cpxxnmi_ioctl(struct inode *inodp, struct file *filep, unsigned int cmd, unsigned long arg);
 ssize_t cpxxnmi_read( struct file *filep, char *buf, size_t length, loff_t *wtf );
 void kd_mksound(unsigned int hz, unsigned int ticks);

 #endif
