/*
 *----------------------------------------------------------------------
 * (C) Copyright 2001, 2002
 * Sysgo Real-Time Solutions GmbH.
 * Klein-Winternheim, Germany
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *			 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *												  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
 *----------------------------------------------------------------------
 *
 *----------------------------------------------------------------------
 * i2c-stpc-atlas.c i2c-hw access for STPC ATLAS boards
 *----------------------------------------------------------------------
 *----------------------------------------------------------------------
 * $Revision: 1.7 $
 * $Id: i2c-stpc-atlas.c,v 1.7 2001/08/16 09:31:22 acs Exp $
 *----------------------------------------------------------------------
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>

#include <asm/uaccess.h>

#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>

#define PORT_CRTEXT 0x3d4

static unsigned char PortData;

EXPORT_NO_SYMBOLS;

MODULE_AUTHOR("SYSGO RTS GmbH, Armin Schindler <acs@sysgo.de>");
MODULE_DESCRIPTION("I2C-Bus adapter routines STPC-ATLAS");
MODULE_LICENSE("GPL");

/* ----- local functions ----------------------------------------------	*/

static void bit_atlas_setscl(void *data, int state)
{
	if (state) {
		PortData |= 0x40;
	} else {
		PortData &= ~0x40;
	}
	outb(0x3f, PORT_CRTEXT);
	outb(PortData, PORT_CRTEXT + 1);
}

static void bit_atlas_setsda(void *data, int state)
{
	if (state) {
		PortData |= 0x80;
	} else {
		PortData &= ~0x80;
	}
	outb(0x3f, PORT_CRTEXT);
	outb(PortData, PORT_CRTEXT + 1);
} 

static int bit_atlas_getscl(void *data)
{
	outb(0x3f, PORT_CRTEXT);
	return ( 0 != ( (inb_p(PORT_CRTEXT + 1)) & 0x10 ) );
}

static int bit_atlas_getsda(void *data)
{
	outb(0x3f, PORT_CRTEXT);
	return ( 0 != ( (inb_p(PORT_CRTEXT + 1)) & 0x20 ) );
}

static int __init bit_atlas_init(void)
{
  /* unlock the CRT extended registers */
  /* needed for I2C access */
  outb(0x6, 0x3c4);
  outb(0x57, 0x3c5);

  PortData = 0xff;
  bit_atlas_setsda((void*)PORT_CRTEXT, 1);
  bit_atlas_setscl((void*)PORT_CRTEXT, 1);
	return 0;
}

static int bit_atlas_reg(struct i2c_client *client)
{
	return 0;
}

static int bit_atlas_unreg(struct i2c_client *client)
{
	return 0;
}

static void bit_atlas_inc_use(struct i2c_adapter *adap)
{
	MOD_INC_USE_COUNT;
}

static void bit_atlas_dec_use(struct i2c_adapter *adap)
{
	MOD_DEC_USE_COUNT;
}

static struct i2c_algo_bit_data bit_atlas_data = {
	data:      NULL,
	setsda:    bit_atlas_setsda,
	setscl:    bit_atlas_setscl,
	getsda:    bit_atlas_getsda,
	getscl:    bit_atlas_getscl,
	udelay:    0,
	mdelay:    0,
	timeout:   25
};

static struct i2c_adapter bit_atlas_ops = {
	name:              "STPC-ATLAS",
	id:                0,
	algo:              NULL,
	algo_data:         &bit_atlas_data,
	inc_use:           bit_atlas_inc_use,
	dec_use:           bit_atlas_dec_use,
	client_register:   bit_atlas_reg,
	client_unregister: bit_atlas_unreg,
	data:              (void *) PORT_CRTEXT 
};

int __init i2c_stpc_atlas_init(void)
{
	printk("i2c-stpc-atlas.o: i2c STPC ATLAS hardware module\n");
	if (bit_atlas_init() == 0) {
		if(i2c_bit_add_bus(&bit_atlas_ops) < 0) {
	    printk("i2c-stpc-atlas.o: register error\n");
			return -ENODEV;
    }
	} else {
	  printk("i2c-stpc-atlas.o: init error\n");
		return -ENODEV;
	}
	printk("i2c-stpc-atlas.o: started.\n");
	return 0;
}

void __exit i2c_bitatlas_exit(void)
{
  i2c_bit_del_bus(&bit_atlas_ops);
}

#ifdef MODULE
module_init(i2c_stpc_atlas_init);
#endif
module_exit(i2c_bitatlas_exit);

