/*
 * Project: rtai_cpp - RTAI C++ Framework 
 *
 * File: $Id: bits.cc,v 1.1 2002/03/29 21:26:21 erwin Exp $
 *
 * Copyright: (C) 2001,2002 Erwin Rol <erwin@muffin.org>
 *
 * Licence:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#include "bits.h"

namespace RTAI {

Bits::Bits(){
	rt_printk("Bits::Bits( ) %p\n",this);
	m_Bits = 0;
}

Bits::Bits( unsigned long mask )
{
	rt_printk("Bits::Bits( unsigned long mask=%d) %p\n",mask,this);
	m_Bits = 0;   
	init( mask  );
}

Bits::~Bits()
{
	if( m_Bits != 0 )
		__rt_bits_delete( m_Bits );
}

void Bits::init(unsigned long mask)
{
	m_Bits = __rt_bits_init( mask );
}
	
unsigned long Bits::get_bits(){
	return rt_get_bits( m_Bits );
}

int Bits::reset( unsigned long mask )
{
	return rt_bits_reset( m_Bits, mask );
}

unsigned long Bits::signal( Function setfun, unsigned long masks )
{
	return rt_bits_signal( m_Bits, setfun, masks );
}

int Bits::wait(Function testfun, unsigned long testmasks, 
		Function exitfun, unsigned long exitmasks)
{
	return rt_bits_wait( m_Bits, testfun, testmasks, exitfun, exitmasks );
}

int Bits::wait_if(Function testfun, unsigned long testmasks, 
		  Function exitfun, unsigned long exitmasks)
{
	return rt_bits_wait_if( m_Bits, testfun, testmasks, exitfun, exitmasks );
}

int Bits::wait_until(Function testfun, unsigned long testmasks, 
		     Function exitfun, unsigned long exitmasks, const Count& time)
{
	return rt_bits_wait_until( m_Bits, testfun, testmasks, exitfun, exitmasks, time);
}

int Bits::wait_timed(Function testfun, unsigned long testmasks, 
		     Function exitfun, unsigned long exitmasks, const Count& delay)
{
	return rt_bits_wait_timed( m_Bits, testfun, testmasks, exitfun, exitmasks, delay );
}

};
