/*
 * Project: rtai_cpp - RTAI C++ Framework 
 *
 * File: $Id: iostream.h,v 1.2 2002/03/30 17:24:38 erwin Exp $
 *
 * Copyright: (C) 2001,2002 Erwin Rol <erwin@muffin.org>
 *
 * Licence:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#ifndef __IOSTREAM_H__
#define __IOSTREAM_H__

namespace RTAI {

class ostream;
typedef ostream& (*__omanip)(ostream&);

class ostream {
public:
	ostream();
	virtual ~ostream();
	
	ostream& set_output_on( bool f );

	ostream& put(char c);
	ostream& put(unsigned char c) { return put((char)c); }
	ostream& put(signed char c) { return put((char)c); }

	ostream& write(const char *s);
	ostream& write(const unsigned char *s) { 
		return write((const char*)s);
	}
	ostream& write(const signed char *s) { 
		return write((const char*)s);
	}

	ostream& operator<<( const char* s );
	ostream& operator<<( const void* p );
	
	ostream& operator<<( unsigned short n);
	ostream& operator<<( unsigned int n);
	ostream& operator<<( unsigned long n );
	ostream& operator<<( unsigned long long n);

	ostream& operator<<( short n);
	ostream& operator<<( int n );
	ostream& operator<<( long n );
	ostream& operator<<( long long n);

	ostream& operator<<( float f );
	ostream& operator<<( double f );
	ostream& operator<<( long double f);

	ostream& operator<<(__omanip func) { return (*func)(*this); }
protected:
	bool m_OutputOn;
};

extern ostream& endl( ostream& s );
extern ostream& output_off( ostream& s);
extern ostream& output_on( ostream& s);

extern ostream cerr;
extern ostream cout;

}; // namespace RTAI

#endif
