/*  Modified by ISDCorp 1999  www.isdcorp.com  */
/*
 * OKI MSM7702-01MS - CL-PS7110 CODEC interface
 *
 * Hacked 1999 by Werner Almesberger
 *
 * This is only a test. The driver is supposed to play a sample that is
 * compiled into the kernel. Once this driver works, we can turn it into a
 * "real" audio device.
 */

#include <linux/module.h>
#include <asm/uaccess.h>
#include <asm/arch/hardware.h>	/*Adressmapping*/

#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/io.h>
#include <asm/hardware/ep7212.h>


//#include <linux/sched.h>
//#include <asm/arch/hardware.h>
#include <asm/irq.h>

#include "sound_config.h"

#ifndef SYSCON3_MCPSEL
#	define SYSCON3_MCPSEL SYSCON3_DAISEL
#endif

// groesse des audio read/write buffers
#define dBufferSize (1024*128)


// debug stuff
#define dDfAssert        0x0001
#define dDfRingCheck     0x0002
#define dDfRingLock      0x0004
#define dDfRingWrite     0x0008
#define dDfRingWriteByte 0x0010
#define dDfRingRead      0x0020
#define dDfRingReadByte  0x0040
#define dDfInit          0x0080
#define dDfCleanup       0x0100
#define dDfIrq           0x0200
#define dDfRead          0x0400
#define dDfWrite         0x0800
#define dDfOpen          0x1000
#define dDfClose         0x2000
#define dDfSync          0x4000
#define dDfIoctl         0x8000

#define dDfAll           0xffff


#define dDebug dDfAssert //|dDfInit|dDfOpen|dDfClose|dDfSync|dDfIoctl

//(dDfAssert|dDfRingCheck|dDfRingWrite|dDfRingRead| 
//               dDfInit|dDfCleanup|dDfRead|dDfWrite|dDfOpen|dDfClose|dDfSync)

#define DEBUG(dl,args...) if ( (dDebug) & (dl) ) printk( "<1>" __FUNCTION__ ": " ##args )
#if ( (dDebug) & dDfAssert)
#    define assert(x) if ( !(x) ) printk( "assertion fail at file '%s',  line %d!\n", __FILE__, __LINE__ );
#else
#    define assert(x)
#endif


#define CODEC_POWER	0x00000001


typedef enum
{
  eAsUnknown,
  eAsIdle,      // da passiert nix
  eAsOperating, // es soll gelesen werden
} oki_state_t;


// status
#define dAfInit   1
#define dAfOpen   2
#define dAfRead   4
#define dAfWrite  8
static int oki_flags = 0;

// minor audio dev
static int oki_minor = -1;

static oki_state_t oki_status_irq = eAsUnknown;
static DECLARE_WAIT_QUEUE_HEAD( oki_read_waitq );
static DECLARE_WAIT_QUEUE_HEAD( oki_write_waitq );


//////////////////////////////////////////////////
//                    ring buffer
//////////////////////////////////////////////////


#define dRbNoData 256


typedef struct ring_buffer
{
  u_char *m_buffer_start;
  u_char *m_buffer_end;
  u_long  m_size;

  u_char *m_read;
  u_long  m_read_len;
  
  u_char *m_write;

  spinlock_t m_lock;
} ring_buffer_t;


#if (dDfDebug & dDfRingCheck)
void 
rb_check( ring_buffer_t *rb )
{
  assert( rb );
  assert( rb->m_buffer_start );
  assert( rb->m_buffer_end );
  assert( rb->m_buffer_end > rb->m_buffer_start );
  assert( rb->m_size > 0 );
  assert( rb->m_buffer_end - rb->m_buffer_start == rb->m_size );
  
  assert( rb->m_read_len >= 0 && rb->m_read_len <= rb->m_size );
  assert( rb->m_read >= rb->m_buffer_start && rb->m_read < rb->m_buffer_end );
  assert( rb->m_write >= rb->m_buffer_start && rb->m_write < rb->m_buffer_end );
}
#else
#define rb_check( rb )
#endif


int 
rb_init( ring_buffer_t *rb, int size )
{
  assert( rb );

  rb->m_buffer_start = kmalloc( size, GFP_KERNEL );

  if ( rb->m_buffer_start == 0 )
     {
       assert( 0 );
       return 0;
     }

  rb->m_buffer_end = rb->m_buffer_start + size;
  
  rb->m_size     = size;
  rb->m_read     = rb->m_buffer_start;
  rb->m_read_len = 0;
  rb->m_write    = rb->m_buffer_start;
  rb->m_lock     = SPIN_LOCK_UNLOCKED;

  rb_check( rb );
  
  return 1;
}


void
rb_cleanup( ring_buffer_t *rb )
{
  rb_check( rb );
  
  if ( rb->m_buffer_start )
     {
       kfree( rb->m_buffer_start );
       rb->m_buffer_start = 0;
     }
}


void 
rb_clear( ring_buffer_t *rb )
{
  rb->m_read     = rb->m_buffer_start;
  rb->m_read_len = 0;
  rb->m_write    = rb->m_buffer_start;
}


void
rb_lock( ring_buffer_t *rb, u_long *flags )
{
  DEBUG( dDfRingLock, "\n" );
  rb_check( rb );
  spin_lock_irqsave( &rb->m_lock, *flags );
}


void
rb_unlock( ring_buffer_t *rb, u_long *flags )
{
  DEBUG( dDfRingLock, "\n" );
  rb_check( rb );
  spin_unlock_irqrestore( &rb->m_lock, *flags );
}


int
rb_read( ring_buffer_t *rb, u_char *buffer, int size )
{
  int ntop, nbot;

  DEBUG( dDfRingRead, ">>> r=%04d, rl=%04ld, w=%04d, size=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, size );

  rb_check( rb );

  if ( size > rb->m_read_len )
       size = rb->m_read_len;

  ntop = rb->m_read + size < rb->m_buffer_end ? size : rb->m_buffer_end - rb->m_read;
  nbot = size - ntop;

  if (    ( ntop && copy_to_user( buffer, rb->m_read, ntop) )
       || ( nbot && copy_to_user( buffer+ntop, rb->m_buffer_start, nbot ) ) )
     {
       DEBUG( dDfRingRead, "copy_to_user fail\n" );
       return -EFAULT;
     }

  //memcpy( buffer, rb->m_read, ntop );
  //memcpy( buffer+ntop, rb->m_buffer_start, nbot );

  rb->m_read = nbot ? rb->m_buffer_start + nbot : rb->m_read + ntop;
  rb->m_read_len -= size;
  
  if ( rb->m_read > rb->m_buffer_end )
       rb->m_read -= rb->m_size;

  DEBUG( dDfRingRead, ">>> r=%04d, rl=%04ld, w=%04d, size=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, size );

  rb_check( rb );

  return size;
}


int
rb_write( ring_buffer_t *rb, u_char *buffer, int size )
{
  int ntop, nbot;
  int n1 = 0,n2 = 0;
  
  DEBUG( dDfRingWrite, ">>> r=%04d, rl=%04ld, w=%04d, size=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, size );

  rb_check( rb );

  if ( size > rb->m_size - rb->m_read_len )
       size = rb->m_size - rb->m_read_len;

  if ( size == 0 )
     {
       DEBUG( dDfRingWrite, "buffer full\n" );
       return 0;
     }

  ntop = rb->m_write + size < rb->m_buffer_end ? size : rb->m_buffer_end - rb->m_write;
  nbot = size - ntop;

  DEBUG( dDfRingWrite, "ntop = %d, nbot = %d\n", ntop, nbot );

  if (    ( ntop && (n1=copy_from_user( rb->m_write, buffer, ntop ) ) )
       || ( nbot && (n2=copy_from_user( rb->m_buffer_start, buffer+ntop, nbot ) ) ) )
     {
       DEBUG( dDfRingWrite, "copy_from_user error: %d, %d\n", n1, n2 );
       return -EFAULT;
     }

  //memcpy( buffer, rb->m_write, ntop );
  //memcpy( buffer+ntop, rb->m_buffer_start, nbot );

  rb->m_write = nbot ? rb->m_buffer_start + nbot : rb->m_write + ntop;

  if ( rb->m_write >= rb->m_buffer_end )
       rb->m_write -= rb->m_size;

  rb->m_read_len += size;
  
  DEBUG( dDfRingWrite, "<<< r=%04d, rl=%04ld, w=%04d, size=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, size );

  rb_check( rb );

  return size;
}


u_long
rb_read_byte( ring_buffer_t *rb )
{
  u_long d;

  DEBUG( dDfRingReadByte, ">>> r=%04d, rl=%04ld, w=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start );

  rb_check( rb );

  if ( rb->m_read_len == 0 )
     {
       DEBUG( dDfRingReadByte, "no data\n" );
       return dRbNoData;
     }

  d = *rb->m_read;
  rb->m_read += 1;

  if ( rb->m_read >= rb->m_buffer_end )
       rb->m_read = rb->m_buffer_start;
  
  rb->m_read_len -= 1;

  DEBUG( dDfRingReadByte, "<<< r=%04d, rl=%04ld, w=%04d, d=%ld\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, d );

  rb_check( rb );

  return d;
}


void
rb_write_byte( ring_buffer_t *rb, u_char d )
{
  DEBUG( dDfRingWriteByte, ">>> r=%04d, rl=%04ld, w=%04d, d=%d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write - rb->m_buffer_start, d );

  rb_check( rb );

  // byte schreiben
  *rb->m_write = d;

  // buffer voll
  if ( rb->m_size - rb->m_read_len == 0 )
     {
       DEBUG( dDfRingWriteByte, "byte at %04d lost\n",
	      rb->m_read - rb->m_buffer_start );

       // verdraenge ein byte aus buffer
       rb->m_read += 1;

       if ( rb->m_read >= rb->m_buffer_end )
	    rb->m_read = rb->m_buffer_start;
     }
  else
       rb->m_read_len += 1;
  
  rb->m_write += 1;
  
  if ( rb->m_write >= rb->m_buffer_end )
       rb->m_write = rb->m_buffer_start;

  DEBUG( dDfRingWriteByte, "<<< r=%04d, rl=%04ld, w=%04d\n",
	 rb->m_read - rb->m_buffer_start, rb->m_read_len, 
	 rb->m_write-rb->m_buffer_start );

  rb_check( rb );
}


// buffer fuer lesen un schreiben
static ring_buffer_t buffer_read;
static ring_buffer_t buffer_write;


//////////////////////////////////////////////////
//                    irq
//////////////////////////////////////////////////


static void
oki_irq( int irq, void *dev_id, struct pt_regs *regs )
{
  u_long flags;
  int rdata = 0; // anzahl wirklich gesendeter daten an oki
  int wdata = 0; // anzahl empfangener daten von oki

  //  int z = 0;

  //  save_flags( f );
  // cli();
  
  // read
  while( !(clps_readl( SYSFLG1 ) & SYSFLG1_CRXFE) )
     {
       u_char d = clps_readb( CODR );

       rb_lock( &buffer_read, &flags );
       rb_write_byte( &buffer_read, d );
       rb_unlock( &buffer_read, &flags );
       rdata++;
     }

  // write
  while( !(clps_readl( SYSFLG1 ) & SYSFLG1_CTXFF) )
     {
       u_long d;
       
       rb_lock( &buffer_write, &flags );
       d = rb_read_byte( &buffer_write );
       rb_unlock( &buffer_write, &flags );

       if ( d == dRbNoData )
	    d = 0xff;       // keine daten
       else
	    wdata++;

       clps_writeb( (u_char)d, CODR );
     }

  clps_writeb( 0, COEOI );

  //restore_flags( f );

  //  if ( z )
  //     DEBUG( dDfWrite, "no data %d\n", z );

  if ( rdata )
       wake_up_interruptible( &oki_read_waitq );

  if ( wdata )
       wake_up_interruptible( &oki_write_waitq );

  DEBUG( dDfIrq, "leave\n" );
}


static void
oki_sync( void )
{
  u_long size = 0;
  u_long flags;
  DECLARE_WAITQUEUE( wait, current );

  assert( (oki_flags & (dAfInit|dAfOpen)) == (dAfInit|dAfOpen) );

  rb_lock( &buffer_write, &flags );
  size = buffer_write.m_read_len;
  rb_unlock( &buffer_write, &flags );

  DEBUG( dDfSync, "enter, size = %ld\n", size );

  if ( size == 0 )
     {
       DEBUG( dDfRead, "leave\n" );
       return;
     }

  assert( oki_status_irq = eAsOperating );

  add_wait_queue( &oki_write_waitq, &wait );

  while( size > 0 )
     {
       set_current_state( TASK_INTERRUPTIBLE );

       rb_lock( &buffer_write, &flags );
       size = buffer_write.m_read_len;
       rb_unlock( &buffer_write, &flags );

       if ( size == 0 )
	    break;

       schedule();

       if ( signal_pending( current ) )
	  {
	    DEBUG( dDfSync, "signal_pending\n" );
	    break;
	  }
     }

  current->state = TASK_RUNNING;
  remove_wait_queue( &oki_write_waitq, &wait );

  DEBUG( dDfSync, "leave\n" );
}


static void
oki_irq_open( void )
{
  DEBUG( dDfRead, "enter\n" );

  clps_writel( clps_readl( SYSCON2 ) | SYSCON2_SERSEL, SYSCON2 );
  clps_writel( clps_readl( SYSCON3 ) & ~SYSCON3_MCPSEL, SYSCON3 );
  clps_writel( clps_readl( SYSCON1 ) | SYSCON1_CDENTX|SYSCON1_CDENRX, SYSCON1 );

  clps_writel( clps_readl( PEDDR ) | CODEC_POWER, PEDDR );	// turn on CODEC power
  clps_writel( clps_readl( PEDR  ) | CODEC_POWER, PEDR );	// turn on CODEC power

  oki_status_irq = eAsOperating;

  //oki_irq( 0, 0, 0 );
  enable_irq( IRQ_CSINT );

  DEBUG( dDfRead, "leave\n" );
}


static void
oki_irq_close( void )
{
  DEBUG( dDfRead, "enter\n" );
  disable_irq( IRQ_CSINT );

  clps_writel( clps_readl( SYSCON1 ) & ~SYSCON1_CDENRX, SYSCON1 );
  clps_writel( clps_readl( PEDR    ) & ~CODEC_POWER, PEDR ); //* turn off CODEC power

  oki_status_irq = eAsIdle;

  DEBUG( dDfRead, "leave\n");
}


//////////////////////////////////////////////////
//                    fops
//////////////////////////////////////////////////


static int
oki_open( struct inode *inode, struct file *file )
{
  dev_t minor = MINOR( inode->i_rdev );

  DEBUG( dDfOpen, "enter\n" );

  if ( (oki_minor & ~0x0F ) != ( minor & ~0x0F) )
     {
       DEBUG( dDfOpen, "dev error %d %d\n", oki_minor, minor );
       return -ENODEV;
     }
  
  MOD_INC_USE_COUNT;

  assert( oki_flags & dAfInit );

  // ist schon offen ?
  if ( oki_flags & dAfOpen )
     {
       DEBUG( dDfOpen, "already open\n" );
       MOD_DEC_USE_COUNT;
       return -EBUSY;
     }

  // das device kann AFMT_MU_LAW format 
  if ( (minor & 0xF) != SND_DEV_AUDIO )
       return -ENODEV;

  // init buffers
  if ( rb_init( &buffer_read, dBufferSize ) == 0 )
     {
       DEBUG( dDfOpen, "no read buffer mem\n" );
       MOD_DEC_USE_COUNT;
      return -ENOMEM;
     }

  if ( rb_init( &buffer_write, dBufferSize ) == 0 )
     {
       DEBUG( dDfOpen, "no write buffer mem\n" );
       rb_cleanup( &buffer_read );
       MOD_DEC_USE_COUNT;
       return -ENOMEM;
     }

  oki_status_irq  = eAsIdle;

  if ( file->f_mode & FMODE_READ )
       oki_flags |= dAfRead;

  if ( file->f_mode & FMODE_WRITE )
       oki_flags |= dAfWrite;

  oki_flags |= dAfOpen;

  oki_irq_open();

  DEBUG( dDfOpen, "leave\n" );

  return 0;
}


static int
oki_close(struct inode *inode, struct file *file)
{
  DEBUG( dDfClose, "enter\n" );

  assert( (oki_flags & (dAfInit|dAfOpen)) == (dAfInit|dAfOpen) );

  if ( oki_status_irq == eAsOperating )
     {
       oki_sync();
       oki_irq_close();
     }

  rb_cleanup( &buffer_read );
  rb_cleanup( &buffer_write );

  oki_flags &= ~(dAfRead|dAfWrite|dAfOpen);

  MOD_DEC_USE_COUNT;

  DEBUG( dDfClose, "leave\n" );
  return 0;
}


static ssize_t
oki_read( struct file *file, char *buffer,
		 size_t count, loff_t *ppos )
{
  ssize_t nread, retval = 0;
  u_long flags;
  DECLARE_WAITQUEUE( wait, current );                               

  DEBUG( dDfRead, "enter, len = %ld\n", (u_long)count );

  assert( (oki_flags & (dAfInit|dAfOpen)) == (dAfInit|dAfOpen) );

  if ( (file->f_mode & FMODE_READ) == 0 )
     {
       DEBUG( dDfRead, "error read open\n" );
       return -EINVAL;
     }

  if ( !access_ok( VERIFY_READ, buffer, count ) )
       return -EFAULT;

  if ( count == 0 )
       return 0;

  add_wait_queue( &oki_read_waitq, &wait );

  nread = 0;

  while( count > 0 )
     {
       ssize_t n;
       
       set_current_state( TASK_INTERRUPTIBLE );
       
       rb_lock( &buffer_read, &flags );
       n = (ssize_t)rb_read( &buffer_read, (u_char *)buffer, count );
       rb_unlock( &buffer_read, &flags );

       if ( n < 0 )
	  {
	    retval = n;
	    break;
	  }
       
       if ( n == 0 )
	  {
	    if ( file->f_flags & O_NONBLOCK )
	       {
		 retval = -EAGAIN;
		 break;
	       }

	    schedule();

	    if ( signal_pending( current ) )
	       {
		 retval = -ERESTARTSYS;
		 break;
	       }

	    continue;
	  }

       nread  += n;
       buffer += n;
       count  -= n;
     }

  current->state = TASK_RUNNING;
  remove_wait_queue( &oki_read_waitq, &wait );

  DEBUG( dDfRead, "leave, n = %ld\n", (u_long)nread );

  retval = nread ? nread : retval;
  return retval;
}


static ssize_t 
oki_write( struct file *file, const char *buffer, size_t count, loff_t *ppos)
{
  ssize_t nwrite, retval = 0;
  u_long flags;
  DECLARE_WAITQUEUE( wait, current );

  DEBUG( dDfWrite, "enter, len = %ld\n", (u_long)count );
  assert( oki_status_irq == eAsOperating );

  if ( (file->f_mode & FMODE_WRITE) == 0 )
       return -EINVAL;

  if ( !access_ok( VERIFY_WRITE, buffer, count ) )
       return -EFAULT;

  add_wait_queue( &oki_write_waitq, &wait );

  nwrite = 0;

  while( count > 0 )
     {
       ssize_t n;
      
       set_current_state( TASK_INTERRUPTIBLE );

       rb_lock( &buffer_write, &flags );
       n = (ssize_t)rb_write( &buffer_write, (u_char *)buffer, (int)count );
       rb_unlock( &buffer_write, &flags );

       DEBUG( dDfWrite, "write %d bytes\n", n );

       if ( n < 0 )
	  {
	    retval = n;
	    break;
	  }

       if ( n == 0 )
	  {
	    if ( file->f_flags & O_NONBLOCK )
	       {
		 retval = -EAGAIN;
		 break;
	       }

	    schedule();

	    if ( signal_pending( current ) )
	       {
		 retval = -ERESTARTSYS;
		 break;
	       }

	    continue;
	  }

       nwrite += n;
       buffer += n;
       count  -= n;
     }       

  current->state = TASK_RUNNING;
  remove_wait_queue( &oki_write_waitq, &wait );

  DEBUG( dDfWrite, "leave, n = %ld\n", (u_long)nwrite );

  retval = nwrite ? nwrite : retval;
  return retval;
}


static int 
oki_ioctl( struct inode *inode,
	   struct file *file,
	   unsigned int cmd,
	   unsigned long arg )
{
  unsigned long flags;
  int ival = 0;

  DEBUG( dDfIoctl, "inode=0x%p, file=0x%p, cmd=0x%x, arg=0x%lx\n",
	 inode, file, cmd, arg );

  switch( cmd )
     {
       case OSS_GETVERSION:          // oss version
	    DEBUG( dDfIoctl, "OSS_GETVERSION\n" );
	    ival = SOUND_VERSION;
	    return put_user( ival, (int *)arg );

       case SNDCTL_DSP_GETCAPS:	     // eigenschaften des devs
	    DEBUG( dDfIoctl, "SNDCTL_DSP_GETCAPS\n");
	    ival = DSP_CAP_DUPLEX | DSP_CAP_REALTIME | DSP_CAP_BATCH;
	    return put_user(ival, (int *) arg);

       case SNDCTL_DSP_GETFMTS:	     // unterstuetzte formate
	    DEBUG( dDfIoctl, "SNDCTL_DSP_GETFMTS\n" );
	    ival = AFMT_MU_LAW;
	    return put_user( ival, (int *)arg );

       case SOUND_PCM_READ_RATE:    // ??? daten date ???
	    DEBUG( dDfIoctl, "SOUND_PCM_READ_RATE\n" );
	    ival = 8000;
	    return put_user( ival, (int *)arg );

       case SOUND_PCM_READ_CHANNELS: // anzahl kanaele
	    DEBUG( dDfIoctl, "SOUND_PCM_READ_CHANNELS\n" );
	    ival = 1;
	    return put_user( ival, (int *)arg );

       case SNDCTL_DSP_SPEED:        // daten rate ???
	    //get_user_ret( ival, (int *)arg, -EFAULT );
	    DEBUG( dDfIoctl, "SNDCTL_DSP_SPEED %d\n", ival );

	    if ( ival )
	       {
		 if ( ival != 8000 )
		      return -EINVAL;
	       }
	    else
		 ival = 8000;

	    return put_user( ival, (int *)arg );

/*
       case SNDCTL_DSP_STEREO: // ????
	 	get_user_ret(ival, (int *) arg, -EFAULT);
		DBGX("SNDCTL_DSP_STEREO %d\n", ival);
		if (ival != 0 && ival != 1)
		 	return -EINVAL;
		if (aport->swstate != SW_INITIAL)
			return -EINVAL;
		if (rport)
			rport->sw_channels = ival + 1;
		if (wport)
			wport->sw_channels = ival + 1;
		return put_user(ival, (int *) arg);

	case SNDCTL_DSP_CHANNELS:
		get_user_ret(ival, (int *) arg, -EFAULT);
		DBGX("SNDCTL_DSP_CHANNELS %d\n", ival);
		if (ival != 1 && ival != 2)
			return -EINVAL;
		if (aport->swstate != SW_INITIAL)
			return -EINVAL;
		if (rport)
			rport->sw_channels = ival;
		if (wport)
			wport->sw_channels = ival;
		return put_user(ival, (int *) arg);

	case SNDCTL_DSP_GETBLKSIZE:
		ival = pcm_setup(devc, rport, wport);
		if (ival < 0) {
			DBGX("SNDCTL_DSP_GETBLKSIZE failed, errno %d\n", ival);
			return ival;
		}
		ival = 1 << aport->sw_fragshift;
		DBGX("SNDCTL_DSP_GETBLKSIZE returning %d\n", ival);
		return put_user(ival, (int *) arg);

	case SNDCTL_DSP_SETFRAGMENT:
		get_user_ret(ival, (int *) arg, -EFAULT);
		DBGX("SNDCTL_DSP_SETFRAGMENT %d:%d\n",
		     ival >> 16, ival & 0xFFFF);
		if (aport->swstate != SW_INITIAL)
			return -EINVAL;
		{
			int sw_fragshift = ival & 0xFFFF;
			int sw_subdivshift = aport->sw_subdivshift;
			int hw_fragshift = sw_fragshift - sw_subdivshift;
			int sw_fragcount = (ival >> 16) & 0xFFFF;
			int hw_fragsize;
			if (hw_fragshift < MIN_FRAGSHIFT)
				hw_fragshift = MIN_FRAGSHIFT;
			if (hw_fragshift > MAX_FRAGSHIFT)
				hw_fragshift = MAX_FRAGSHIFT;
			sw_fragshift = hw_fragshift + aport->sw_subdivshift;
			hw_fragsize = 1 << hw_fragshift;
			if (sw_fragcount < MIN_FRAGCOUNT(hw_fragsize))
				sw_fragcount = MIN_FRAGCOUNT(hw_fragsize);
			if (sw_fragcount > MAX_FRAGCOUNT(hw_fragsize))
				sw_fragcount = MAX_FRAGCOUNT(hw_fragsize);
			DBGPV("sw_fragshift = %d\n", sw_fragshift);
			DBGPV("rport = 0x%p, wport = 0x%p\n", rport, wport);
			if (rport) {
				rport->sw_fragshift = sw_fragshift;
				rport->sw_fragcount = sw_fragcount;
			}
			if (wport) {
				wport->sw_fragshift = sw_fragshift;
				wport->sw_fragcount = sw_fragcount;
			}
			ival = sw_fragcount << 16 | sw_fragshift;
		}
		DBGX("SNDCTL_DSP_SETFRAGMENT returns %d:%d\n",
		      ival >> 16, ival & 0xFFFF);
		return put_user(ival, (int *) arg);

	case SNDCTL_DSP_SUBDIVIDE:
                get_user_ret(ival, (int *) arg, -EFAULT);
		DBGX("SNDCTL_DSP_SUBDIVIDE %d\n", ival);
		if (aport->swstate != SW_INITIAL)
			return -EINVAL;
		{
			int subdivshift;
			int hw_fragshift, hw_fragsize, hw_fragcount;
			switch (ival) {
			case 1: subdivshift = 0; break;
			case 2: subdivshift = 1; break;
			case 4: subdivshift = 2; break;
			default: return -EINVAL;
			}
			hw_fragshift = aport->sw_fragshift - subdivshift;
			if (hw_fragshift < MIN_FRAGSHIFT ||
			    hw_fragshift > MAX_FRAGSHIFT)
				return -EINVAL;
			hw_fragsize = 1 << hw_fragshift;
			hw_fragcount = aport->sw_fragcount >> subdivshift;
			if (hw_fragcount < MIN_FRAGCOUNT(hw_fragsize) ||
			    hw_fragcount > MAX_FRAGCOUNT(hw_fragsize))
				return -EINVAL;
			if (rport)
				rport->sw_subdivshift = subdivshift;
			if (wport)
				wport->sw_subdivshift = subdivshift;
		}
		return 0;
*/

       case SNDCTL_DSP_SETFMT:	// fromat
	    //get_user_ret( ival, (int *)arg, -EFAULT );
	    DEBUG( dDfIoctl, "SNDCTL_DSP_SETFMT %d\n", ival );

	    if ( ival != AFMT_QUERY )
	       {
		 if ( ival != AFMT_MU_LAW )
		      return -EINVAL;
	       }
	    else
		 ival = AFMT_MU_LAW;

	    return put_user( ival, (int *)arg );

/*
	case SNDCTL_DSP_GETOSPACE:
		DBGXV("SNDCTL_DSP_GETOSPACE\n");
		if (!wport)
			return -EINVAL;
		ival = pcm_setup(devc, rport, wport);
		if (ival < 0)
			return ival;
		ival = swb_inc_u(wport, 0);
		buf_info.fragments = ival >> wport->sw_fragshift;
		buf_info.fragstotal = wport->sw_fragcount;
		buf_info.fragsize = 1 << wport->sw_fragshift;
		buf_info.bytes = ival;
		DBGXV("SNDCTL_DSP_GETOSPACE returns { %d %d %d %d }\n",
		     buf_info.fragments, buf_info.fragstotal,
		     buf_info.fragsize, buf_info.bytes);
		return copy_to_user((void *) arg, &buf_info, sizeof buf_info);

	case SNDCTL_DSP_GETISPACE:
		DBGX("SNDCTL_DSP_GETISPACE\n");
		if (!rport)
			return -EINVAL;
		ival = pcm_setup(devc, rport, wport);
		if (ival < 0)
			return ival;
		ival = swb_inc_u(rport, 0);
		buf_info.fragments = ival >> rport->sw_fragshift;
		buf_info.fragstotal = rport->sw_fragcount;
		buf_info.fragsize = 1 << rport->sw_fragshift;
		buf_info.bytes = ival;
		DBGX("SNDCTL_DSP_GETISPACE returns { %d %d %d %d }\n",
		     buf_info.fragments, buf_info.fragstotal,
		     buf_info.fragsize, buf_info.bytes);
		return copy_to_user((void *) arg, &buf_info, sizeof buf_info);
*/

       case SNDCTL_DSP_NONBLOCK:
	    DEBUG( dDfIoctl, "SNDCTL_DSP_NONBLOCK\n" );
	    file->f_flags |= O_NONBLOCK;
	    return 0;

       case SNDCTL_DSP_RESET:		/* _SIO  ('P', 0) */
	    DEBUG( dDfIoctl, "SNDCTL_DSP_RESET\n" );

	    // losche alle buffer
	    rb_lock( &buffer_read, &flags );
	    rb_clear( &buffer_read );
	    rb_unlock( &buffer_read, &flags );

	    rb_lock( &buffer_write, &flags );
	    rb_clear( &buffer_write );
	    rb_unlock( &buffer_write, &flags );

	    return 0;

	case SNDCTL_DSP_SYNC:		/* _SIO  ('P', 1) */	     
	     DEBUG( dDfIoctl, "SNDCTL_DSP_SYNC\n");
	     oki_sync();
	     return 0;

/*
	case SNDCTL_DSP_POST:
		DBGX("SNDCTL_DSP_POST\n");
		if (!wport)
			return -EINVAL;
		pcm_flush_frag(devc);
		return 0;

	case SNDCTL_DSP_GETIPTR:
		DBGX("SNDCTL_DSP_GETIPTR\n");
		if (!rport)
			return -EINVAL;
		spin_lock_irqsave(&rport->lock, flags);
		{
			ustmsc_t ustmsc;
			if (rport->hwstate == HW_RUNNING) {
				ASSERT(rport->swstate == SW_RUN);
				li_read_USTMSC(&rport->chan, &ustmsc);
				info.bytes = ustmsc.msc - rport->MSC_offset;
				info.bytes *= rport->frame_size;
			} else {
				info.bytes = rport->byte_count;
			}
			info.blocks = rport->frag_count;
			info.ptr = 0;	// not implemented
			rport->frag_count = 0;
		}
		spin_unlock_irqrestore(&rport->lock, flags);
		return copy_to_user((void *) arg, &info, sizeof info);

	case SNDCTL_DSP_GETOPTR:	// _SIOR ('P',18, count_info)
		DBGX("SNDCTL_DSP_GETOPTR\n");
		if (!wport)
			return -EINVAL;
		spin_lock_irqsave(&wport->lock, flags);
		{
			ustmsc_t ustmsc;
			if (wport->hwstate == HW_RUNNING) {
				ASSERT(wport->swstate == SW_RUN);
				li_read_USTMSC(&wport->chan, &ustmsc);
				info.bytes = ustmsc.msc - wport->MSC_offset;
				info.bytes *= wport->frame_size;
			} else {
				info.bytes = wport->byte_count;
			}
			info.blocks = wport->frag_count;
			info.ptr = 0;	// not implemented
			wport->frag_count = 0;
		}
		spin_unlock_irqrestore(&wport->lock, flags);
		return copy_to_user((void *) arg, &info, sizeof info);

	case SNDCTL_DSP_GETODELAY:	// _SIOR ('P', 23, int)
		DBGX("SNDCTL_DSP_GETODELAY\n");
		if (!wport)
			return -EINVAL;
		spin_lock_irqsave(&wport->lock, flags);
		{
			int fsize = wport->frame_size;
			ival = wport->swb_i_avail / fsize;
			if (wport->hwstate == HW_RUNNING) {
				int swptr, hwptr, hwframes, hwbytes, hwsize;
				int totalhwbytes;
				ustmsc_t ustmsc;

				hwsize = wport->hwbuf_size;
				swptr = li_read_swptr(&wport->chan);
				li_read_USTMSC(&wport->chan, &ustmsc);
				hwframes = ustmsc.msc - wport->MSC_offset;
				totalhwbytes = hwframes * fsize;
				hwptr = totalhwbytes % hwsize;
				hwbytes = (swptr - hwptr + hwsize) % hwsize;
				ival += hwbytes / fsize;
			}
		}
		spin_unlock_irqrestore(&wport->lock, flags);
		return put_user(ival, (int *) arg);
*/

	case SNDCTL_DSP_PROFILE:	/* _SIOW ('P', 23, int) */
	     DEBUG( dDfIoctl, "SNDCTL_DSP_PROFILE\n" );

		/*
		 * Thomas Sailer explains SNDCTL_DSP_PROFILE
		 * (private email, March 24, 1999):
		 *
		 *     This gives the sound driver a hint on what it
		 *     should do with partial fragments
		 *     (i.e. fragments partially filled with write).
		 *     This can direct the driver to zero them or
		 *     leave them alone.  But don't ask me what this
		 *     is good for, my driver just zeroes the last
		 *     fragment before the receiver stops, no idea
		 *     what good for any other behaviour could
		 *     be. Implementing it as NOP seems safe.
		 */

		break;

/*
       case SNDCTL_DSP_GETTRIGGER:	// _SIOR ('P',16, int)
		DBGX("SNDCTL_DSP_GETTRIGGER\n");
		ival = 0;
		if (rport) {
			spin_lock_irqsave(&rport->lock, flags);
			{
				if (!(rport->flags & DISABLED))
					ival |= PCM_ENABLE_INPUT;
			}
			spin_unlock_irqrestore(&rport->lock, flags);
		}
		if (wport) {
			spin_lock_irqsave(&wport->lock, flags);
			{
				if (!(wport->flags & DISABLED))
					ival |= PCM_ENABLE_OUTPUT;
			}
			spin_unlock_irqrestore(&wport->lock, flags);
		}
		return put_user(ival, (int *) arg);

	case SNDCTL_DSP_SETTRIGGER:	// _SIOW ('P',16, int)
		get_user_ret(ival, (int *) arg, -EFAULT);
		DBGX("SNDCTL_DSP_SETTRIGGER %d\n", ival);

		// If user is disabling I/O and port is not in initial
		// state, fail with EINVAL.

		if (((rport && !(ival & PCM_ENABLE_INPUT)) ||
		     (wport && !(ival & PCM_ENABLE_OUTPUT))) &&
		    aport->swstate != SW_INITIAL)
			return -EINVAL;

		if (rport) {
			vwsnd_port_hwstate_t hwstate;
			spin_lock_irqsave(&rport->lock, flags);
			{
				hwstate = rport->hwstate;
				if (ival & PCM_ENABLE_INPUT)
					rport->flags &= ~DISABLED;
				else
					rport->flags |= DISABLED;
			}
			spin_unlock_irqrestore(&rport->lock, flags);
			if (hwstate != HW_RUNNING && ival & PCM_ENABLE_INPUT) {

				if (rport->swstate == SW_INITIAL)
					pcm_setup(devc, rport, wport);
				else
					li_activate_dma(&rport->chan);
			}
		}
		if (wport) {
			vwsnd_port_flags_t pflags;
			spin_lock_irqsave(&wport->lock, flags);
			{
				pflags = wport->flags;
				if (ival & PCM_ENABLE_OUTPUT)
					wport->flags &= ~DISABLED;
				else
					wport->flags |= DISABLED;
			}
			spin_unlock_irqrestore(&wport->lock, flags);
			if (pflags & DISABLED && ival & PCM_ENABLE_OUTPUT) {
				if (wport->swstate == SW_RUN)
					pcm_output(devc, 0, 0);
			}
		}
		return 0;
*/

       default:
	    DEBUG( dDfIoctl, "unknown ioctl 0x%x\n", cmd );
	    return -EINVAL;
     }

  DEBUG( dDfIoctl, "unimplemented ioctl 0x%x\n", cmd );
  return -EINVAL;
}


static struct file_operations oki_fops =
{
  owner:	THIS_MODULE,
  open:		oki_open,
  release:	oki_close,
  read:         oki_read,
  write:	oki_write,
  ioctl:        oki_ioctl
};


int __init
init_oki( void )
{
  int rv;
  
  DEBUG( dDfInit, "enter\n" );

  oki_flags       = 0;
  oki_status_irq  = eAsUnknown;

  disable_irq( IRQ_CSINT );

  rv = request_irq( IRQ_CSINT, oki_irq, SA_SHIRQ, "codec", oki_irq );
  
  if ( rv )
     {
       printk( KERN_ERR "oki sound: unable to allocate interrupt: %d\n", rv );
       return -EIO;
     }

  disable_irq( IRQ_CSINT );

  oki_minor = register_sound_dsp( &oki_fops, -1 );

  if ( oki_minor < 0 )
     {
       printk( KERN_ERR "oki sound: register_sound_dsp error %d\n", oki_minor );
       free_irq( IRQ_CSINT, NULL );
       return oki_minor;
     }

  oki_flags = dAfInit;

  DEBUG( dDfInit, "leave\n" );

  printk( "<1>oki sound\n" );

  return 0;
}


static void __init
cleanup_oki( void )
{
  DEBUG( dDfCleanup, "enter\n" );
  assert( oki_flags == dAfInit );

  assert( oki_minor >= 0 );
  unregister_sound_dsp( oki_minor );

  free_irq( IRQ_CSINT, NULL );
  oki_minor = -1;

  DEBUG( dDfCleanup, "leave\n" );
}


#if LINUX_VERSION_CODE >= 0x20211
module_init( init_oki );
module_exit( cleanup_oki );
#endif
